# Copyright 2018 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
# pylint: disable=line-too-long
"""List of renames to apply when converting from TF 1.0 to TF 2.0.

THIS FILE IS AUTOGENERATED: To update, please run:
  bazel build tensorflow/tools/compatibility/update:generate_v2_renames_map
  bazel-bin/tensorflow/tools/compatibility/update/generate_v2_renames_map
pyformat --in_place third_party/tensorflow/tools/compatibility/renames_v2.py
This file should be updated whenever endpoints are deprecated.
"""
renames = {
    'tf.AUTO_REUSE':
        'tf.compat.v1.AUTO_REUSE',
    'tf.AttrValue':
        'tf.compat.v1.AttrValue',
    'tf.COMPILER_VERSION':
        'tf.version.COMPILER_VERSION',
    'tf.CXX11_ABI_FLAG':
        'tf.sysconfig.CXX11_ABI_FLAG',
    'tf.ConditionalAccumulator':
        'tf.compat.v1.ConditionalAccumulator',
    'tf.ConditionalAccumulatorBase':
        'tf.compat.v1.ConditionalAccumulatorBase',
    'tf.ConfigProto':
        'tf.compat.v1.ConfigProto',
    'tf.Dimension':
        'tf.compat.v1.Dimension',
    'tf.Event':
        'tf.compat.v1.Event',
    'tf.FIFOQueue':
        'tf.queue.FIFOQueue',
    'tf.FixedLenFeature':
        'tf.io.FixedLenFeature',
    'tf.FixedLenSequenceFeature':
        'tf.io.FixedLenSequenceFeature',
    'tf.FixedLengthRecordReader':
        'tf.compat.v1.FixedLengthRecordReader',
    'tf.GIT_VERSION':
        'tf.version.GIT_VERSION',
    'tf.GPUOptions':
        'tf.compat.v1.GPUOptions',
    'tf.GRAPH_DEF_VERSION':
        'tf.version.GRAPH_DEF_VERSION',
    'tf.GRAPH_DEF_VERSION_MIN_CONSUMER':
        'tf.version.GRAPH_DEF_VERSION_MIN_CONSUMER',
    'tf.GRAPH_DEF_VERSION_MIN_PRODUCER':
        'tf.version.GRAPH_DEF_VERSION_MIN_PRODUCER',
    'tf.GraphDef':
        'tf.compat.v1.GraphDef',
    'tf.GraphKeys':
        'tf.compat.v1.GraphKeys',
    'tf.GraphOptions':
        'tf.compat.v1.GraphOptions',
    'tf.HistogramProto':
        'tf.compat.v1.HistogramProto',
    'tf.IdentityReader':
        'tf.compat.v1.IdentityReader',
    'tf.InteractiveSession':
        'tf.compat.v1.InteractiveSession',
    'tf.LMDBReader':
        'tf.compat.v1.LMDBReader',
    'tf.LogMessage':
        'tf.compat.v1.LogMessage',
    'tf.MONOLITHIC_BUILD':
        'tf.sysconfig.MONOLITHIC_BUILD',
    'tf.MetaGraphDef':
        'tf.compat.v1.MetaGraphDef',
    'tf.NameAttrList':
        'tf.compat.v1.NameAttrList',
    'tf.NoGradient':
        'tf.no_gradient',
    'tf.NodeDef':
        'tf.compat.v1.NodeDef',
    'tf.NotDifferentiable':
        'tf.no_gradient',
    'tf.OpError':
        'tf.errors.OpError',
    'tf.OptimizerOptions':
        'tf.compat.v1.OptimizerOptions',
    'tf.PaddingFIFOQueue':
        'tf.queue.PaddingFIFOQueue',
    'tf.Print':
        'tf.compat.v1.Print',
    'tf.PriorityQueue':
        'tf.queue.PriorityQueue',
    'tf.QUANTIZED_DTYPES':
        'tf.dtypes.QUANTIZED_DTYPES',
    'tf.QueueBase':
        'tf.queue.QueueBase',
    'tf.RandomShuffleQueue':
        'tf.queue.RandomShuffleQueue',
    'tf.ReaderBase':
        'tf.compat.v1.ReaderBase',
    'tf.RunMetadata':
        'tf.compat.v1.RunMetadata',
    'tf.RunOptions':
        'tf.compat.v1.RunOptions',
    'tf.Session':
        'tf.compat.v1.Session',
    'tf.SessionLog':
        'tf.compat.v1.SessionLog',
    'tf.SparseConditionalAccumulator':
        'tf.compat.v1.SparseConditionalAccumulator',
    'tf.SparseFeature':
        'tf.io.SparseFeature',
    'tf.SparseTensorValue':
        'tf.compat.v1.SparseTensorValue',
    'tf.Summary':
        'tf.compat.v1.Summary',
    'tf.SummaryMetadata':
        'tf.compat.v1.SummaryMetadata',
    'tf.TFRecordReader':
        'tf.compat.v1.TFRecordReader',
    'tf.TensorInfo':
        'tf.compat.v1.TensorInfo',
    'tf.TextLineReader':
        'tf.compat.v1.TextLineReader',
    'tf.VERSION':
        'tf.version.VERSION',
    'tf.VarLenFeature':
        'tf.io.VarLenFeature',
    'tf.VariableScope':
        'tf.compat.v1.VariableScope',
    'tf.WholeFileReader':
        'tf.compat.v1.WholeFileReader',
    'tf.accumulate_n':
        'tf.math.accumulate_n',
    'tf.add_check_numerics_ops':
        'tf.compat.v1.add_check_numerics_ops',
    'tf.add_to_collection':
        'tf.compat.v1.add_to_collection',
    'tf.add_to_collections':
        'tf.compat.v1.add_to_collections',
    'tf.all_variables':
        'tf.compat.v1.all_variables',
    'tf.angle':
        'tf.math.angle',
    'tf.app.run':
        'tf.compat.v1.app.run',
    'tf.assert_greater_equal':
        'tf.compat.v1.assert_greater_equal',
    'tf.assert_integer':
        'tf.debugging.assert_integer',
    'tf.assert_less_equal':
        'tf.compat.v1.assert_less_equal',
    'tf.assert_near':
        'tf.compat.v1.assert_near',
    'tf.assert_negative':
        'tf.compat.v1.assert_negative',
    'tf.assert_non_negative':
        'tf.compat.v1.assert_non_negative',
    'tf.assert_non_positive':
        'tf.compat.v1.assert_non_positive',
    'tf.assert_none_equal':
        'tf.compat.v1.assert_none_equal',
    'tf.assert_positive':
        'tf.compat.v1.assert_positive',
    'tf.assert_proper_iterable':
        'tf.debugging.assert_proper_iterable',
    'tf.assert_rank_at_least':
        'tf.compat.v1.assert_rank_at_least',
    'tf.assert_rank_in':
        'tf.compat.v1.assert_rank_in',
    'tf.assert_same_float_dtype':
        'tf.debugging.assert_same_float_dtype',
    'tf.assert_scalar':
        'tf.compat.v1.assert_scalar',
    'tf.assert_type':
        'tf.debugging.assert_type',
    'tf.assert_variables_initialized':
        'tf.compat.v1.assert_variables_initialized',
    'tf.assign':
        'tf.compat.v1.assign',
    'tf.assign_add':
        'tf.compat.v1.assign_add',
    'tf.assign_sub':
        'tf.compat.v1.assign_sub',
    'tf.batch_scatter_update':
        'tf.compat.v1.batch_scatter_update',
    'tf.betainc':
        'tf.math.betainc',
    'tf.ceil':
        'tf.math.ceil',
    'tf.check_numerics':
        'tf.debugging.check_numerics',
    'tf.cholesky':
        'tf.linalg.cholesky',
    'tf.cholesky_solve':
        'tf.linalg.cholesky_solve',
    'tf.clip_by_average_norm':
        'tf.compat.v1.clip_by_average_norm',
    'tf.colocate_with':
        'tf.compat.v1.colocate_with',
    'tf.conj':
        'tf.math.conj',
    'tf.container':
        'tf.compat.v1.container',
    'tf.control_flow_v2_enabled':
        'tf.compat.v1.control_flow_v2_enabled',
    'tf.convert_to_tensor_or_indexed_slices':
        'tf.compat.v1.convert_to_tensor_or_indexed_slices',
    'tf.convert_to_tensor_or_sparse_tensor':
        'tf.compat.v1.convert_to_tensor_or_sparse_tensor',
    'tf.count_up_to':
        'tf.compat.v1.count_up_to',
    'tf.create_partitioned_variables':
        'tf.compat.v1.create_partitioned_variables',
    'tf.cross':
        'tf.linalg.cross',
    'tf.cumprod':
        'tf.math.cumprod',
    'tf.data.get_output_classes':
        'tf.compat.v1.data.get_output_classes',
    'tf.data.get_output_shapes':
        'tf.compat.v1.data.get_output_shapes',
    'tf.data.get_output_types':
        'tf.compat.v1.data.get_output_types',
    'tf.data.make_initializable_iterator':
        'tf.compat.v1.data.make_initializable_iterator',
    'tf.data.make_one_shot_iterator':
        'tf.compat.v1.data.make_one_shot_iterator',
    'tf.debugging.is_finite':
        'tf.math.is_finite',
    'tf.debugging.is_inf':
        'tf.math.is_inf',
    'tf.debugging.is_nan':
        'tf.math.is_nan',
    'tf.debugging.is_non_decreasing':
        'tf.math.is_non_decreasing',
    'tf.debugging.is_strictly_increasing':
        'tf.math.is_strictly_increasing',
    'tf.decode_base64':
        'tf.io.decode_base64',
    'tf.decode_compressed':
        'tf.io.decode_compressed',
    'tf.decode_json_example':
        'tf.io.decode_json_example',
    'tf.delete_session_tensor':
        'tf.compat.v1.delete_session_tensor',
    'tf.depth_to_space':
        'tf.compat.v1.depth_to_space',
    'tf.dequantize':
        'tf.quantization.dequantize',
    'tf.deserialize_many_sparse':
        'tf.io.deserialize_many_sparse',
    'tf.diag':
        'tf.linalg.tensor_diag',
    'tf.diag_part':
        'tf.linalg.tensor_diag_part',
    'tf.digamma':
        'tf.math.digamma',
    'tf.dimension_at_index':
        'tf.compat.dimension_at_index',
    'tf.dimension_value':
        'tf.compat.dimension_value',
    'tf.disable_control_flow_v2':
        'tf.compat.v1.disable_control_flow_v2',
    'tf.disable_eager_execution':
        'tf.compat.v1.disable_eager_execution',
    'tf.disable_resource_variables':
        'tf.compat.v1.disable_resource_variables',
    'tf.disable_tensor_equality':
        'tf.compat.v1.disable_tensor_equality',
    'tf.disable_v2_behavior':
        'tf.compat.v1.disable_v2_behavior',
    'tf.disable_v2_tensorshape':
        'tf.compat.v1.disable_v2_tensorshape',
    'tf.distribute.experimental.ParameterServerStrategy':
        'tf.compat.v1.distribute.experimental.ParameterServerStrategy',
    'tf.distribute.get_loss_reduction':
        'tf.compat.v1.distribute.get_loss_reduction',
    'tf.distributions.Bernoulli':
        'tf.compat.v1.distributions.Bernoulli',
    'tf.distributions.Beta':
        'tf.compat.v1.distributions.Beta',
    'tf.distributions.Categorical':
        'tf.compat.v1.distributions.Categorical',
    'tf.distributions.Dirichlet':
        'tf.compat.v1.distributions.Dirichlet',
    'tf.distributions.DirichletMultinomial':
        'tf.compat.v1.distributions.DirichletMultinomial',
    'tf.distributions.Distribution':
        'tf.compat.v1.distributions.Distribution',
    'tf.distributions.Exponential':
        'tf.compat.v1.distributions.Exponential',
    'tf.distributions.FULLY_REPARAMETERIZED':
        'tf.compat.v1.distributions.FULLY_REPARAMETERIZED',
    'tf.distributions.Gamma':
        'tf.compat.v1.distributions.Gamma',
    'tf.distributions.Laplace':
        'tf.compat.v1.distributions.Laplace',
    'tf.distributions.Multinomial':
        'tf.compat.v1.distributions.Multinomial',
    'tf.distributions.NOT_REPARAMETERIZED':
        'tf.compat.v1.distributions.NOT_REPARAMETERIZED',
    'tf.distributions.Normal':
        'tf.compat.v1.distributions.Normal',
    'tf.distributions.RegisterKL':
        'tf.compat.v1.distributions.RegisterKL',
    'tf.distributions.ReparameterizationType':
        'tf.compat.v1.distributions.ReparameterizationType',
    'tf.distributions.StudentT':
        'tf.compat.v1.distributions.StudentT',
    'tf.distributions.Uniform':
        'tf.compat.v1.distributions.Uniform',
    'tf.distributions.kl_divergence':
        'tf.compat.v1.distributions.kl_divergence',
    'tf.div':
        'tf.compat.v1.div',
    'tf.div_no_nan':
        'tf.math.divide_no_nan',
    'tf.dtypes.as_string':
        'tf.strings.as_string',
    'tf.enable_control_flow_v2':
        'tf.compat.v1.enable_control_flow_v2',
    'tf.enable_eager_execution':
        'tf.compat.v1.enable_eager_execution',
    'tf.enable_resource_variables':
        'tf.compat.v1.enable_resource_variables',
    'tf.enable_tensor_equality':
        'tf.compat.v1.enable_tensor_equality',
    'tf.enable_v2_behavior':
        'tf.compat.v1.enable_v2_behavior',
    'tf.enable_v2_tensorshape':
        'tf.compat.v1.enable_v2_tensorshape',
    'tf.encode_base64':
        'tf.io.encode_base64',
    'tf.erf':
        'tf.math.erf',
    'tf.erfc':
        'tf.math.erfc',
    'tf.estimator.experimental.KMeans':
        'tf.compat.v1.estimator.experimental.KMeans',
    'tf.estimator.experimental.dnn_logit_fn_builder':
        'tf.compat.v1.estimator.experimental.dnn_logit_fn_builder',
    'tf.estimator.experimental.linear_logit_fn_builder':
        'tf.compat.v1.estimator.experimental.linear_logit_fn_builder',
    'tf.estimator.inputs.numpy_input_fn':
        'tf.compat.v1.estimator.inputs.numpy_input_fn',
    'tf.estimator.inputs.pandas_input_fn':
        'tf.compat.v1.estimator.inputs.pandas_input_fn',
    'tf.estimator.tpu.InputPipelineConfig':
        'tf.compat.v1.estimator.tpu.InputPipelineConfig',
    'tf.estimator.tpu.RunConfig':
        'tf.compat.v1.estimator.tpu.RunConfig',
    'tf.estimator.tpu.TPUConfig':
        'tf.compat.v1.estimator.tpu.TPUConfig',
    'tf.estimator.tpu.TPUEstimator':
        'tf.compat.v1.estimator.tpu.TPUEstimator',
    'tf.estimator.tpu.TPUEstimatorSpec':
        'tf.compat.v1.estimator.tpu.TPUEstimatorSpec',
    'tf.estimator.tpu.experimental.EmbeddingConfigSpec':
        'tf.compat.v1.estimator.tpu.experimental.EmbeddingConfigSpec',
    'tf.executing_eagerly_outside_functions':
        'tf.compat.v1.executing_eagerly_outside_functions',
    'tf.experimental.output_all_intermediates':
        'tf.compat.v1.experimental.output_all_intermediates',
    'tf.expm1':
        'tf.math.expm1',
    'tf.fake_quant_with_min_max_args':
        'tf.quantization.fake_quant_with_min_max_args',
    'tf.fake_quant_with_min_max_args_gradient':
        'tf.quantization.fake_quant_with_min_max_args_gradient',
    'tf.fake_quant_with_min_max_vars':
        'tf.quantization.fake_quant_with_min_max_vars',
    'tf.fake_quant_with_min_max_vars_gradient':
        'tf.quantization.fake_quant_with_min_max_vars_gradient',
    'tf.fake_quant_with_min_max_vars_per_channel':
        'tf.quantization.fake_quant_with_min_max_vars_per_channel',
    'tf.fake_quant_with_min_max_vars_per_channel_gradient':
        'tf.quantization.fake_quant_with_min_max_vars_per_channel_gradient',
    'tf.feature_column.input_layer':
        'tf.compat.v1.feature_column.input_layer',
    'tf.feature_column.linear_model':
        'tf.compat.v1.feature_column.linear_model',
    'tf.feature_column.shared_embedding_columns':
        'tf.compat.v1.feature_column.shared_embedding_columns',
    'tf.fft':
        'tf.signal.fft',
    'tf.fft2d':
        'tf.signal.fft2d',
    'tf.fft3d':
        'tf.signal.fft3d',
    'tf.fixed_size_partitioner':
        'tf.compat.v1.fixed_size_partitioner',
    'tf.floordiv':
        'tf.math.floordiv',
    'tf.floormod':
        'tf.math.floormod',
    'tf.get_collection':
        'tf.compat.v1.get_collection',
    'tf.get_collection_ref':
        'tf.compat.v1.get_collection_ref',
    'tf.get_default_graph':
        'tf.compat.v1.get_default_graph',
    'tf.get_default_session':
        'tf.compat.v1.get_default_session',
    'tf.get_local_variable':
        'tf.compat.v1.get_local_variable',
    'tf.get_seed':
        'tf.compat.v1.get_seed',
    'tf.get_session_handle':
        'tf.compat.v1.get_session_handle',
    'tf.get_session_tensor':
        'tf.compat.v1.get_session_tensor',
    'tf.get_variable':
        'tf.compat.v1.get_variable',
    'tf.get_variable_scope':
        'tf.compat.v1.get_variable_scope',
    'tf.gfile.FastGFile':
        'tf.compat.v1.gfile.FastGFile',
    'tf.global_norm':
        'tf.linalg.global_norm',
    'tf.global_variables':
        'tf.compat.v1.global_variables',
    'tf.global_variables_initializer':
        'tf.compat.v1.global_variables_initializer',
    'tf.graph_util.convert_variables_to_constants':
        'tf.compat.v1.graph_util.convert_variables_to_constants',
    'tf.graph_util.extract_sub_graph':
        'tf.compat.v1.graph_util.extract_sub_graph',
    'tf.graph_util.must_run_on_cpu':
        'tf.compat.v1.graph_util.must_run_on_cpu',
    'tf.graph_util.remove_training_nodes':
        'tf.compat.v1.graph_util.remove_training_nodes',
    'tf.graph_util.tensor_shape_from_node_def_name':
        'tf.compat.v1.graph_util.tensor_shape_from_node_def_name',
    'tf.ifft':
        'tf.signal.ifft',
    'tf.ifft2d':
        'tf.signal.ifft2d',
    'tf.ifft3d':
        'tf.signal.ifft3d',
    'tf.igamma':
        'tf.math.igamma',
    'tf.igammac':
        'tf.math.igammac',
    'tf.imag':
        'tf.math.imag',
    'tf.image.resize_area':
        'tf.compat.v1.image.resize_area',
    'tf.image.resize_bicubic':
        'tf.compat.v1.image.resize_bicubic',
    'tf.image.resize_bilinear':
        'tf.compat.v1.image.resize_bilinear',
    'tf.image.resize_image_with_crop_or_pad':
        'tf.image.resize_with_crop_or_pad',
    'tf.image.resize_image_with_pad':
        'tf.compat.v1.image.resize_image_with_pad',
    'tf.image.resize_nearest_neighbor':
        'tf.compat.v1.image.resize_nearest_neighbor',
    'tf.image.transpose_image':
        'tf.image.transpose',
    'tf.initialize_all_tables':
        'tf.compat.v1.initialize_all_tables',
    'tf.initialize_all_variables':
        'tf.compat.v1.initialize_all_variables',
    'tf.initialize_local_variables':
        'tf.compat.v1.initialize_local_variables',
    'tf.initialize_variables':
        'tf.compat.v1.initialize_variables',
    'tf.initializers.global_variables':
        'tf.compat.v1.initializers.global_variables',
    'tf.initializers.local_variables':
        'tf.compat.v1.initializers.local_variables',
    'tf.initializers.tables_initializer':
        'tf.compat.v1.initializers.tables_initializer',
    'tf.initializers.uniform_unit_scaling':
        'tf.compat.v1.initializers.uniform_unit_scaling',
    'tf.initializers.variables':
        'tf.compat.v1.initializers.variables',
    'tf.invert_permutation':
        'tf.math.invert_permutation',
    'tf.io.PaddingFIFOQueue':
        'tf.queue.PaddingFIFOQueue',
    'tf.io.PriorityQueue':
        'tf.queue.PriorityQueue',
    'tf.io.QueueBase':
        'tf.queue.QueueBase',
    'tf.io.RandomShuffleQueue':
        'tf.queue.RandomShuffleQueue',
    'tf.io.TFRecordCompressionType':
        'tf.compat.v1.io.TFRecordCompressionType',
    'tf.io.tf_record_iterator':
        'tf.compat.v1.io.tf_record_iterator',
    'tf.is_finite':
        'tf.math.is_finite',
    'tf.is_inf':
        'tf.math.is_inf',
    'tf.is_nan':
        'tf.math.is_nan',
    'tf.is_non_decreasing':
        'tf.math.is_non_decreasing',
    'tf.is_numeric_tensor':
        'tf.debugging.is_numeric_tensor',
    'tf.is_strictly_increasing':
        'tf.math.is_strictly_increasing',
    'tf.is_variable_initialized':
        'tf.compat.v1.is_variable_initialized',
    'tf.keras.backend.get_session':
        'tf.compat.v1.keras.backend.get_session',
    'tf.keras.backend.set_session':
        'tf.compat.v1.keras.backend.set_session',
    'tf.keras.experimental.export_saved_model':
        'tf.compat.v1.keras.experimental.export_saved_model',
    'tf.keras.experimental.load_from_saved_model':
        'tf.compat.v1.keras.experimental.load_from_saved_model',
    'tf.keras.layers.CuDNNGRU':
        'tf.compat.v1.keras.layers.CuDNNGRU',
    'tf.keras.layers.CuDNNLSTM':
        'tf.compat.v1.keras.layers.CuDNNLSTM',
    'tf.keras.layers.disable_v2_dtype_behavior':
        'tf.compat.v1.keras.layers.disable_v2_dtype_behavior',
    'tf.keras.layers.enable_v2_dtype_behavior':
        'tf.compat.v1.keras.layers.enable_v2_dtype_behavior',
    'tf.keras.losses.cosine':
        'tf.keras.losses.cosine_similarity',
    'tf.keras.losses.cosine_proximity':
        'tf.keras.losses.cosine_similarity',
    'tf.keras.metrics.cosine':
        'tf.keras.losses.cosine_similarity',
    'tf.keras.metrics.cosine_proximity':
        'tf.keras.losses.cosine_similarity',
    'tf.keras.models.LinearModel':
        'tf.compat.v1.keras.models.LinearModel',
    'tf.keras.models.WideDeepModel':
        'tf.compat.v1.keras.models.WideDeepModel',
    'tf.keras.utils.DeterministicRandomTestTool':
        'tf.compat.v1.keras.utils.DeterministicRandomTestTool',
    'tf.keras.utils.track_tf1_style_variables':
        'tf.compat.v1.keras.utils.track_tf1_style_variables',
    'tf.keras.utils.get_or_create_layer':
        'tf.compat.v1.keras.utils.get_or_create_layer',
    'tf.layers.AveragePooling1D':
        'tf.compat.v1.layers.AveragePooling1D',
    'tf.layers.AveragePooling2D':
        'tf.compat.v1.layers.AveragePooling2D',
    'tf.layers.AveragePooling3D':
        'tf.compat.v1.layers.AveragePooling3D',
    'tf.layers.BatchNormalization':
        'tf.compat.v1.layers.BatchNormalization',
    'tf.layers.Conv1D':
        'tf.compat.v1.layers.Conv1D',
    'tf.layers.Conv2D':
        'tf.compat.v1.layers.Conv2D',
    'tf.layers.Conv2DTranspose':
        'tf.compat.v1.layers.Conv2DTranspose',
    'tf.layers.Conv3D':
        'tf.compat.v1.layers.Conv3D',
    'tf.layers.Conv3DTranspose':
        'tf.compat.v1.layers.Conv3DTranspose',
    'tf.layers.Dense':
        'tf.compat.v1.layers.Dense',
    'tf.layers.Dropout':
        'tf.compat.v1.layers.Dropout',
    'tf.layers.Flatten':
        'tf.compat.v1.layers.Flatten',
    'tf.layers.InputSpec':
        'tf.keras.layers.InputSpec',
    'tf.layers.Layer':
        'tf.compat.v1.layers.Layer',
    'tf.layers.MaxPooling1D':
        'tf.compat.v1.layers.MaxPooling1D',
    'tf.layers.MaxPooling2D':
        'tf.compat.v1.layers.MaxPooling2D',
    'tf.layers.MaxPooling3D':
        'tf.compat.v1.layers.MaxPooling3D',
    'tf.layers.SeparableConv1D':
        'tf.compat.v1.layers.SeparableConv1D',
    'tf.layers.SeparableConv2D':
        'tf.compat.v1.layers.SeparableConv2D',
    'tf.layers.average_pooling1d':
        'tf.compat.v1.layers.average_pooling1d',
    'tf.layers.average_pooling2d':
        'tf.compat.v1.layers.average_pooling2d',
    'tf.layers.average_pooling3d':
        'tf.compat.v1.layers.average_pooling3d',
    'tf.layers.batch_normalization':
        'tf.compat.v1.layers.batch_normalization',
    'tf.layers.conv1d':
        'tf.compat.v1.layers.conv1d',
    'tf.layers.conv2d':
        'tf.compat.v1.layers.conv2d',
    'tf.layers.conv2d_transpose':
        'tf.compat.v1.layers.conv2d_transpose',
    'tf.layers.conv3d':
        'tf.compat.v1.layers.conv3d',
    'tf.layers.conv3d_transpose':
        'tf.compat.v1.layers.conv3d_transpose',
    'tf.layers.dense':
        'tf.compat.v1.layers.dense',
    'tf.layers.dropout':
        'tf.compat.v1.layers.dropout',
    'tf.layers.experimental.keras_style_scope':
        'tf.compat.v1.layers.experimental.keras_style_scope',
    'tf.layers.experimental.set_keras_style':
        'tf.compat.v1.layers.experimental.set_keras_style',
    'tf.layers.flatten':
        'tf.compat.v1.layers.flatten',
    'tf.layers.max_pooling1d':
        'tf.compat.v1.layers.max_pooling1d',
    'tf.layers.max_pooling2d':
        'tf.compat.v1.layers.max_pooling2d',
    'tf.layers.max_pooling3d':
        'tf.compat.v1.layers.max_pooling3d',
    'tf.layers.separable_conv1d':
        'tf.compat.v1.layers.separable_conv1d',
    'tf.layers.separable_conv2d':
        'tf.compat.v1.layers.separable_conv2d',
    'tf.lbeta':
        'tf.math.lbeta',
    'tf.lgamma':
        'tf.math.lgamma',
    'tf.lin_space':
        'tf.linspace',
    'tf.linalg.transpose':
        'tf.linalg.matrix_transpose',
    'tf.lite.OpHint':
        'tf.compat.v1.lite.OpHint',
    'tf.lite.TocoConverter':
        'tf.compat.v1.lite.TocoConverter',
    'tf.lite.constants.FLOAT16':
        'tf.compat.v1.lite.constants.FLOAT16',
    'tf.lite.constants.GRAPHVIZ_DOT':
        'tf.compat.v1.lite.constants.GRAPHVIZ_DOT',
    'tf.lite.constants.INT8':
        'tf.compat.v1.lite.constants.INT8',
    'tf.lite.constants.TFLITE':
        'tf.compat.v1.lite.constants.TFLITE',
    'tf.lite.experimental.convert_op_hints_to_stubs':
        'tf.compat.v1.lite.experimental.convert_op_hints_to_stubs',
    'tf.lite.toco_convert':
        'tf.compat.v1.lite.toco_convert',
    'tf.local_variables':
        'tf.compat.v1.local_variables',
    'tf.local_variables_initializer':
        'tf.compat.v1.local_variables_initializer',
    'tf.log':
        'tf.math.log',
    'tf.log1p':
        'tf.math.log1p',
    'tf.log_sigmoid':
        'tf.math.log_sigmoid',
    'tf.logging.DEBUG':
        'tf.compat.v1.logging.DEBUG',
    'tf.logging.ERROR':
        'tf.compat.v1.logging.ERROR',
    'tf.logging.FATAL':
        'tf.compat.v1.logging.FATAL',
    'tf.logging.INFO':
        'tf.compat.v1.logging.INFO',
    'tf.logging.TaskLevelStatusMessage':
        'tf.compat.v1.logging.TaskLevelStatusMessage',
    'tf.logging.WARN':
        'tf.compat.v1.logging.WARN',
    'tf.logging.debug':
        'tf.compat.v1.logging.debug',
    'tf.logging.error':
        'tf.compat.v1.logging.error',
    'tf.logging.fatal':
        'tf.compat.v1.logging.fatal',
    'tf.logging.flush':
        'tf.compat.v1.logging.flush',
    'tf.logging.get_verbosity':
        'tf.compat.v1.logging.get_verbosity',
    'tf.logging.info':
        'tf.compat.v1.logging.info',
    'tf.logging.log':
        'tf.compat.v1.logging.log',
    'tf.logging.log_every_n':
        'tf.compat.v1.logging.log_every_n',
    'tf.logging.log_first_n':
        'tf.compat.v1.logging.log_first_n',
    'tf.logging.log_if':
        'tf.compat.v1.logging.log_if',
    'tf.logging.set_verbosity':
        'tf.compat.v1.logging.set_verbosity',
    'tf.logging.vlog':
        'tf.compat.v1.logging.vlog',
    'tf.logging.warn':
        'tf.compat.v1.logging.warn',
    'tf.logging.warning':
        'tf.compat.v1.logging.warning',
    'tf.logical_xor':
        'tf.math.logical_xor',
    'tf.losses.Reduction':
        'tf.compat.v1.losses.Reduction',
    'tf.losses.absolute_difference':
        'tf.compat.v1.losses.absolute_difference',
    'tf.losses.add_loss':
        'tf.compat.v1.losses.add_loss',
    'tf.losses.compute_weighted_loss':
        'tf.compat.v1.losses.compute_weighted_loss',
    'tf.losses.cosine_distance':
        'tf.compat.v1.losses.cosine_distance',
    'tf.losses.get_losses':
        'tf.compat.v1.losses.get_losses',
    'tf.losses.get_regularization_loss':
        'tf.compat.v1.losses.get_regularization_loss',
    'tf.losses.get_regularization_losses':
        'tf.compat.v1.losses.get_regularization_losses',
    'tf.losses.get_total_loss':
        'tf.compat.v1.losses.get_total_loss',
    'tf.losses.hinge_loss':
        'tf.compat.v1.losses.hinge_loss',
    'tf.losses.huber_loss':
        'tf.compat.v1.losses.huber_loss',
    'tf.losses.log_loss':
        'tf.compat.v1.losses.log_loss',
    'tf.losses.mean_pairwise_squared_error':
        'tf.compat.v1.losses.mean_pairwise_squared_error',
    'tf.losses.mean_squared_error':
        'tf.compat.v1.losses.mean_squared_error',
    'tf.losses.sigmoid_cross_entropy':
        'tf.compat.v1.losses.sigmoid_cross_entropy',
    'tf.losses.softmax_cross_entropy':
        'tf.compat.v1.losses.softmax_cross_entropy',
    'tf.losses.sparse_softmax_cross_entropy':
        'tf.compat.v1.losses.sparse_softmax_cross_entropy',
    'tf.make_template':
        'tf.compat.v1.make_template',
    'tf.manip.gather_nd':
        'tf.compat.v1.manip.gather_nd',
    'tf.manip.reshape':
        'tf.reshape',
    'tf.manip.reverse':
        'tf.reverse',
    'tf.manip.roll':
        'tf.roll',
    'tf.manip.scatter_nd':
        'tf.scatter_nd',
    'tf.manip.space_to_batch_nd':
        'tf.space_to_batch_nd',
    'tf.manip.tile':
        'tf.tile',
    'tf.matching_files':
        'tf.io.matching_files',
    'tf.matrix_band_part':
        'tf.linalg.band_part',
    'tf.matrix_determinant':
        'tf.linalg.det',
    'tf.matrix_diag':
        'tf.linalg.diag',
    'tf.matrix_diag_part':
        'tf.linalg.diag_part',
    'tf.matrix_inverse':
        'tf.linalg.inv',
    'tf.matrix_set_diag':
        'tf.linalg.set_diag',
    'tf.matrix_solve':
        'tf.linalg.solve',
    'tf.matrix_solve_ls':
        'tf.linalg.lstsq',
    'tf.matrix_transpose':
        'tf.linalg.matrix_transpose',
    'tf.matrix_triangular_solve':
        'tf.linalg.triangular_solve',
    'tf.mixed_precision.DynamicLossScale':
        'tf.compat.v1.mixed_precision.DynamicLossScale',
    'tf.mixed_precision.FixedLossScale':
        'tf.compat.v1.mixed_precision.FixedLossScale',
    'tf.mixed_precision.LossScale':
        'tf.compat.v1.mixed_precision.LossScale',
    'tf.mixed_precision.experimental.DynamicLossScale':
        'tf.compat.v1.mixed_precision.experimental.DynamicLossScale',
    'tf.mixed_precision.experimental.FixedLossScale':
        'tf.compat.v1.mixed_precision.experimental.FixedLossScale',
    'tf.mixed_precision.experimental.LossScale':
        'tf.compat.v1.mixed_precision.experimental.LossScale',
    'tf.metrics.accuracy':
        'tf.compat.v1.metrics.accuracy',
    'tf.metrics.auc':
        'tf.compat.v1.metrics.auc',
    'tf.metrics.average_precision_at_k':
        'tf.compat.v1.metrics.average_precision_at_k',
    'tf.metrics.false_negatives':
        'tf.compat.v1.metrics.false_negatives',
    'tf.metrics.false_negatives_at_thresholds':
        'tf.compat.v1.metrics.false_negatives_at_thresholds',
    'tf.metrics.false_positives':
        'tf.compat.v1.metrics.false_positives',
    'tf.metrics.false_positives_at_thresholds':
        'tf.compat.v1.metrics.false_positives_at_thresholds',
    'tf.metrics.mean':
        'tf.compat.v1.metrics.mean',
    'tf.metrics.mean_absolute_error':
        'tf.compat.v1.metrics.mean_absolute_error',
    'tf.metrics.mean_cosine_distance':
        'tf.compat.v1.metrics.mean_cosine_distance',
    'tf.metrics.mean_iou':
        'tf.compat.v1.metrics.mean_iou',
    'tf.metrics.mean_per_class_accuracy':
        'tf.compat.v1.metrics.mean_per_class_accuracy',
    'tf.metrics.mean_relative_error':
        'tf.compat.v1.metrics.mean_relative_error',
    'tf.metrics.mean_squared_error':
        'tf.compat.v1.metrics.mean_squared_error',
    'tf.metrics.mean_tensor':
        'tf.compat.v1.metrics.mean_tensor',
    'tf.metrics.percentage_below':
        'tf.compat.v1.metrics.percentage_below',
    'tf.metrics.precision':
        'tf.compat.v1.metrics.precision',
    'tf.metrics.precision_at_k':
        'tf.compat.v1.metrics.precision_at_k',
    'tf.metrics.precision_at_thresholds':
        'tf.compat.v1.metrics.precision_at_thresholds',
    'tf.metrics.precision_at_top_k':
        'tf.compat.v1.metrics.precision_at_top_k',
    'tf.metrics.recall':
        'tf.compat.v1.metrics.recall',
    'tf.metrics.recall_at_k':
        'tf.compat.v1.metrics.recall_at_k',
    'tf.metrics.recall_at_thresholds':
        'tf.compat.v1.metrics.recall_at_thresholds',
    'tf.metrics.recall_at_top_k':
        'tf.compat.v1.metrics.recall_at_top_k',
    'tf.metrics.root_mean_squared_error':
        'tf.compat.v1.metrics.root_mean_squared_error',
    'tf.metrics.sensitivity_at_specificity':
        'tf.compat.v1.metrics.sensitivity_at_specificity',
    'tf.metrics.sparse_average_precision_at_k':
        'tf.compat.v1.metrics.sparse_average_precision_at_k',
    'tf.metrics.sparse_precision_at_k':
        'tf.compat.v1.metrics.sparse_precision_at_k',
    'tf.metrics.specificity_at_sensitivity':
        'tf.compat.v1.metrics.specificity_at_sensitivity',
    'tf.metrics.true_negatives':
        'tf.compat.v1.metrics.true_negatives',
    'tf.metrics.true_negatives_at_thresholds':
        'tf.compat.v1.metrics.true_negatives_at_thresholds',
    'tf.metrics.true_positives':
        'tf.compat.v1.metrics.true_positives',
    'tf.metrics.true_positives_at_thresholds':
        'tf.compat.v1.metrics.true_positives_at_thresholds',
    'tf.min_max_variable_partitioner':
        'tf.compat.v1.min_max_variable_partitioner',
    'tf.mixed_precision.MixedPrecisionLossScaleOptimizer':
        'tf.compat.v1.mixed_precision.MixedPrecisionLossScaleOptimizer',
    'tf.mixed_precision.disable_mixed_precision_graph_rewrite':
        'tf.compat.v1.mixed_precision.disable_mixed_precision_graph_rewrite',
    'tf.mixed_precision.enable_mixed_precision_graph_rewrite':
        'tf.compat.v1.mixed_precision.enable_mixed_precision_graph_rewrite',
    'tf.mod':
        'tf.math.floormod',
    'tf.model_variables':
        'tf.compat.v1.model_variables',
    'tf.moving_average_variables':
        'tf.compat.v1.moving_average_variables',
    'tf.nn.avg_pool_v2':
        'tf.nn.avg_pool',
    'tf.nn.bidirectional_dynamic_rnn':
        'tf.compat.v1.nn.bidirectional_dynamic_rnn',
    'tf.nn.conv2d_backprop_filter':
        'tf.compat.v1.nn.conv2d_backprop_filter',
    'tf.nn.conv3d_backprop_filter':
        'tf.compat.v1.nn.conv3d_backprop_filter',
    'tf.nn.conv3d_backprop_filter_v2':
        'tf.compat.v1.nn.conv3d_backprop_filter_v2',
    'tf.nn.ctc_beam_search_decoder_v2':
        'tf.nn.ctc_beam_search_decoder',
    'tf.nn.ctc_loss_v2':
        'tf.compat.v1.nn.ctc_loss_v2',
    'tf.nn.depthwise_conv2d_native':
        'tf.compat.v1.nn.depthwise_conv2d_native',
    'tf.nn.depthwise_conv2d_native_backprop_filter':
        'tf.nn.depthwise_conv2d_backprop_filter',
    'tf.nn.depthwise_conv2d_native_backprop_input':
        'tf.nn.depthwise_conv2d_backprop_input',
    'tf.nn.dynamic_rnn':
        'tf.compat.v1.nn.dynamic_rnn',
    'tf.nn.log_uniform_candidate_sampler':
        'tf.random.log_uniform_candidate_sampler',
    'tf.nn.max_pool_v2':
        'tf.nn.max_pool',
    'tf.nn.quantized_avg_pool':
        'tf.compat.v1.nn.quantized_avg_pool',
    'tf.nn.quantized_conv2d':
        'tf.compat.v1.nn.quantized_conv2d',
    'tf.nn.quantized_max_pool':
        'tf.compat.v1.nn.quantized_max_pool',
    'tf.nn.quantized_relu_x':
        'tf.compat.v1.nn.quantized_relu_x',
    'tf.nn.raw_rnn':
        'tf.compat.v1.nn.raw_rnn',
    'tf.nn.relu_layer':
        'tf.compat.v1.nn.relu_layer',
    'tf.nn.rnn_cell.BasicLSTMCell':
        'tf.compat.v1.nn.rnn_cell.BasicLSTMCell',
    'tf.nn.rnn_cell.BasicRNNCell':
        'tf.compat.v1.nn.rnn_cell.BasicRNNCell',
    'tf.nn.rnn_cell.DeviceWrapper':
        'tf.compat.v1.nn.rnn_cell.DeviceWrapper',
    'tf.nn.rnn_cell.DropoutWrapper':
        'tf.compat.v1.nn.rnn_cell.DropoutWrapper',
    'tf.nn.rnn_cell.GRUCell':
        'tf.compat.v1.nn.rnn_cell.GRUCell',
    'tf.nn.rnn_cell.LSTMCell':
        'tf.compat.v1.nn.rnn_cell.LSTMCell',
    'tf.nn.rnn_cell.LSTMStateTuple':
        'tf.compat.v1.nn.rnn_cell.LSTMStateTuple',
    'tf.nn.rnn_cell.MultiRNNCell':
        'tf.compat.v1.nn.rnn_cell.MultiRNNCell',
    'tf.nn.rnn_cell.RNNCell':
        'tf.compat.v1.nn.rnn_cell.RNNCell',
    'tf.nn.rnn_cell.ResidualWrapper':
        'tf.compat.v1.nn.rnn_cell.ResidualWrapper',
    'tf.nn.static_bidirectional_rnn':
        'tf.compat.v1.nn.static_bidirectional_rnn',
    'tf.nn.static_rnn':
        'tf.compat.v1.nn.static_rnn',
    'tf.nn.static_state_saving_rnn':
        'tf.compat.v1.nn.static_state_saving_rnn',
    'tf.nn.uniform_candidate_sampler':
        'tf.random.uniform_candidate_sampler',
    'tf.nn.xw_plus_b':
        'tf.compat.v1.nn.xw_plus_b',
    'tf.no_regularizer':
        'tf.compat.v1.no_regularizer',
    'tf.op_scope':
        'tf.compat.v1.op_scope',
    'tf.parse_single_sequence_example':
        'tf.io.parse_single_sequence_example',
    'tf.parse_tensor':
        'tf.io.parse_tensor',
    'tf.placeholder':
        'tf.compat.v1.placeholder',
    'tf.placeholder_with_default':
        'tf.compat.v1.placeholder_with_default',
    'tf.polygamma':
        'tf.math.polygamma',
    'tf.profiler.AdviceProto':
        'tf.compat.v1.profiler.AdviceProto',
    'tf.profiler.GraphNodeProto':
        'tf.compat.v1.profiler.GraphNodeProto',
    'tf.profiler.MultiGraphNodeProto':
        'tf.compat.v1.profiler.MultiGraphNodeProto',
    'tf.profiler.OpLogProto':
        'tf.compat.v1.profiler.OpLogProto',
    'tf.profiler.ProfileOptionBuilder':
        'tf.compat.v1.profiler.ProfileOptionBuilder',
    'tf.profiler.Profiler':
        'tf.compat.v1.profiler.Profiler',
    'tf.profiler.advise':
        'tf.compat.v1.profiler.advise',
    'tf.profiler.profile':
        'tf.compat.v1.profiler.profile',
    'tf.profiler.write_op_log':
        'tf.compat.v1.profiler.write_op_log',
    'tf.py_func':
        'tf.compat.v1.py_func',
    'tf.python_io.TFRecordCompressionType':
        'tf.compat.v1.python_io.TFRecordCompressionType',
    'tf.python_io.TFRecordOptions':
        'tf.io.TFRecordOptions',
    'tf.python_io.TFRecordWriter':
        'tf.io.TFRecordWriter',
    'tf.python_io.tf_record_iterator':
        'tf.compat.v1.python_io.tf_record_iterator',
    'tf.qr':
        'tf.linalg.qr',
    'tf.quantize':
        'tf.quantization.quantize',
    'tf.quantized_concat':
        'tf.quantization.quantized_concat',
    'tf.ragged.RaggedTensorValue':
        'tf.compat.v1.ragged.RaggedTensorValue',
    'tf.ragged.constant_value':
        'tf.compat.v1.ragged.constant_value',
    'tf.ragged.placeholder':
        'tf.compat.v1.ragged.placeholder',
    'tf.random.get_seed':
        'tf.compat.v1.random.get_seed',
    'tf.random.set_random_seed':
        'tf.compat.v1.random.set_random_seed',
    'tf.random_crop':
        'tf.image.random_crop',
    'tf.random_gamma':
        'tf.random.gamma',
    'tf.random_normal':
        'tf.random.normal',
    'tf.random_shuffle':
        'tf.random.shuffle',
    'tf.random_uniform':
        'tf.random.uniform',
    'tf.read_file':
        'tf.io.read_file',
    'tf.real':
        'tf.math.real',
    'tf.reciprocal':
        'tf.math.reciprocal',
    'tf.regex_replace':
        'tf.strings.regex_replace',
    'tf.report_uninitialized_variables':
        'tf.compat.v1.report_uninitialized_variables',
    'tf.reset_default_graph':
        'tf.compat.v1.reset_default_graph',
    'tf.resource_loader.get_data_files_path':
        'tf.compat.v1.resource_loader.get_data_files_path',
    'tf.resource_loader.get_path_to_datafile':
        'tf.compat.v1.resource_loader.get_path_to_datafile',
    'tf.resource_loader.get_root_dir_with_all_resources':
        'tf.compat.v1.resource_loader.get_root_dir_with_all_resources',
    'tf.resource_loader.load_resource':
        'tf.compat.v1.resource_loader.load_resource',
    'tf.resource_loader.readahead_file_path':
        'tf.compat.v1.resource_loader.readahead_file_path',
    'tf.resource_variables_enabled':
        'tf.compat.v1.resource_variables_enabled',
    'tf.reverse_v2':
        'tf.reverse',
    'tf.rint':
        'tf.math.rint',
    'tf.rsqrt':
        'tf.math.rsqrt',
    'tf.saved_model.Builder':
        'tf.compat.v1.saved_model.Builder',
    'tf.saved_model.LEGACY_INIT_OP_KEY':
        'tf.compat.v1.saved_model.LEGACY_INIT_OP_KEY',
    'tf.saved_model.MAIN_OP_KEY':
        'tf.compat.v1.saved_model.MAIN_OP_KEY',
    'tf.saved_model.build_signature_def':
        'tf.compat.v1.saved_model.build_signature_def',
    'tf.saved_model.build_tensor_info':
        'tf.compat.v1.saved_model.build_tensor_info',
    'tf.saved_model.builder.SavedModelBuilder':
        'tf.compat.v1.saved_model.builder.SavedModelBuilder',
    'tf.saved_model.classification_signature_def':
        'tf.compat.v1.saved_model.classification_signature_def',
    'tf.saved_model.constants.ASSETS_DIRECTORY':
        'tf.saved_model.ASSETS_DIRECTORY',
    'tf.saved_model.constants.ASSETS_KEY':
        'tf.saved_model.ASSETS_KEY',
    'tf.saved_model.constants.DEBUG_DIRECTORY':
        'tf.saved_model.DEBUG_DIRECTORY',
    'tf.saved_model.constants.DEBUG_INFO_FILENAME_PB':
        'tf.saved_model.DEBUG_INFO_FILENAME_PB',
    'tf.saved_model.constants.LEGACY_INIT_OP_KEY':
        'tf.compat.v1.saved_model.constants.LEGACY_INIT_OP_KEY',
    'tf.saved_model.constants.MAIN_OP_KEY':
        'tf.compat.v1.saved_model.constants.MAIN_OP_KEY',
    'tf.saved_model.constants.SAVED_MODEL_FILENAME_PB':
        'tf.saved_model.SAVED_MODEL_FILENAME_PB',
    'tf.saved_model.constants.SAVED_MODEL_FILENAME_PBTXT':
        'tf.saved_model.SAVED_MODEL_FILENAME_PBTXT',
    'tf.saved_model.constants.SAVED_MODEL_SCHEMA_VERSION':
        'tf.saved_model.SAVED_MODEL_SCHEMA_VERSION',
    'tf.saved_model.constants.VARIABLES_DIRECTORY':
        'tf.saved_model.VARIABLES_DIRECTORY',
    'tf.saved_model.constants.VARIABLES_FILENAME':
        'tf.saved_model.VARIABLES_FILENAME',
    'tf.saved_model.experimental.save':
        'tf.saved_model.save',
    'tf.saved_model.get_tensor_from_tensor_info':
        'tf.compat.v1.saved_model.get_tensor_from_tensor_info',
    'tf.saved_model.is_valid_signature':
        'tf.compat.v1.saved_model.is_valid_signature',
    'tf.saved_model.loader.load':
        'tf.compat.v1.saved_model.loader.load',
    'tf.saved_model.loader.maybe_saved_model_directory':
        'tf.compat.v1.saved_model.loader.maybe_saved_model_directory',
    'tf.saved_model.main_op.main_op':
        'tf.compat.v1.saved_model.main_op.main_op',
    'tf.saved_model.main_op.main_op_with_restore':
        'tf.compat.v1.saved_model.main_op.main_op_with_restore',
    'tf.saved_model.main_op_with_restore':
        'tf.compat.v1.saved_model.main_op_with_restore',
    'tf.saved_model.maybe_saved_model_directory':
        'tf.compat.v1.saved_model.maybe_saved_model_directory',
    'tf.saved_model.predict_signature_def':
        'tf.compat.v1.saved_model.predict_signature_def',
    'tf.saved_model.regression_signature_def':
        'tf.compat.v1.saved_model.regression_signature_def',
    'tf.saved_model.signature_constants.CLASSIFY_INPUTS':
        'tf.saved_model.CLASSIFY_INPUTS',
    'tf.saved_model.signature_constants.CLASSIFY_METHOD_NAME':
        'tf.saved_model.CLASSIFY_METHOD_NAME',
    'tf.saved_model.signature_constants.CLASSIFY_OUTPUT_CLASSES':
        'tf.saved_model.CLASSIFY_OUTPUT_CLASSES',
    'tf.saved_model.signature_constants.CLASSIFY_OUTPUT_SCORES':
        'tf.saved_model.CLASSIFY_OUTPUT_SCORES',
    'tf.saved_model.signature_constants.DEFAULT_SERVING_SIGNATURE_DEF_KEY':
        'tf.saved_model.DEFAULT_SERVING_SIGNATURE_DEF_KEY',
    'tf.saved_model.signature_constants.PREDICT_INPUTS':
        'tf.saved_model.PREDICT_INPUTS',
    'tf.saved_model.signature_constants.PREDICT_METHOD_NAME':
        'tf.saved_model.PREDICT_METHOD_NAME',
    'tf.saved_model.signature_constants.PREDICT_OUTPUTS':
        'tf.saved_model.PREDICT_OUTPUTS',
    'tf.saved_model.signature_constants.REGRESS_INPUTS':
        'tf.saved_model.REGRESS_INPUTS',
    'tf.saved_model.signature_constants.REGRESS_METHOD_NAME':
        'tf.saved_model.REGRESS_METHOD_NAME',
    'tf.saved_model.signature_constants.REGRESS_OUTPUTS':
        'tf.saved_model.REGRESS_OUTPUTS',
    'tf.saved_model.signature_def_utils.MethodNameUpdater':
        'tf.compat.v1.saved_model.signature_def_utils.MethodNameUpdater',
    'tf.saved_model.signature_def_utils.build_signature_def':
        'tf.compat.v1.saved_model.signature_def_utils.build_signature_def',
    'tf.saved_model.signature_def_utils.classification_signature_def':
        'tf.compat.v1.saved_model.signature_def_utils.classification_signature_def',
    'tf.saved_model.signature_def_utils.is_valid_signature':
        'tf.compat.v1.saved_model.signature_def_utils.is_valid_signature',
    'tf.saved_model.signature_def_utils.predict_signature_def':
        'tf.compat.v1.saved_model.signature_def_utils.predict_signature_def',
    'tf.saved_model.signature_def_utils.regression_signature_def':
        'tf.compat.v1.saved_model.signature_def_utils.regression_signature_def',
    'tf.saved_model.simple_save':
        'tf.compat.v1.saved_model.simple_save',
    'tf.saved_model.tag_constants.GPU':
        'tf.saved_model.GPU',
    'tf.saved_model.tag_constants.SERVING':
        'tf.saved_model.SERVING',
    'tf.saved_model.tag_constants.TPU':
        'tf.saved_model.TPU',
    'tf.saved_model.tag_constants.TRAINING':
        'tf.saved_model.TRAINING',
    'tf.saved_model.utils.build_tensor_info':
        'tf.compat.v1.saved_model.utils.build_tensor_info',
    'tf.saved_model.utils.get_tensor_from_tensor_info':
        'tf.compat.v1.saved_model.utils.get_tensor_from_tensor_info',
    'tf.scatter_add':
        'tf.compat.v1.scatter_add',
    'tf.scatter_div':
        'tf.compat.v1.scatter_div',
    'tf.scatter_max':
        'tf.compat.v1.scatter_max',
    'tf.scatter_min':
        'tf.compat.v1.scatter_min',
    'tf.scatter_mul':
        'tf.compat.v1.scatter_mul',
    'tf.scatter_nd_add':
        'tf.compat.v1.scatter_nd_add',
    'tf.scatter_nd_sub':
        'tf.compat.v1.scatter_nd_sub',
    'tf.scatter_nd_max':
        'tf.compat.v1.scatter_nd_max',
    'tf.scatter_nd_min':
        'tf.compat.v1.scatter_nd_min',
    'tf.scatter_nd_update':
        'tf.compat.v1.scatter_nd_update',
    'tf.scatter_sub':
        'tf.compat.v1.scatter_sub',
    'tf.scatter_update':
        'tf.compat.v1.scatter_update',
    'tf.segment_max':
        'tf.math.segment_max',
    'tf.segment_mean':
        'tf.math.segment_mean',
    'tf.segment_min':
        'tf.math.segment_min',
    'tf.segment_prod':
        'tf.math.segment_prod',
    'tf.segment_sum':
        'tf.math.segment_sum',
    'tf.self_adjoint_eig':
        'tf.linalg.eigh',
    'tf.self_adjoint_eigvals':
        'tf.linalg.eigvalsh',
    'tf.serialize_many_sparse':
        'tf.compat.v1.serialize_many_sparse',
    'tf.serialize_sparse':
        'tf.compat.v1.serialize_sparse',
    'tf.serialize_tensor':
        'tf.io.serialize_tensor',
    'tf.set_random_seed':
        'tf.compat.v1.set_random_seed',
    'tf.setdiff1d':
        'tf.compat.v1.setdiff1d',
    'tf.sets.set_difference':
        'tf.sets.difference',
    'tf.sets.set_intersection':
        'tf.sets.intersection',
    'tf.sets.set_size':
        'tf.sets.size',
    'tf.sets.set_union':
        'tf.sets.union',
    'tf.space_to_depth':
        'tf.compat.v1.space_to_depth',
    'tf.sparse.SparseConditionalAccumulator':
        'tf.compat.v1.sparse.SparseConditionalAccumulator',
    'tf.sparse.matmul':
        'tf.sparse.sparse_dense_matmul',
    'tf.sparse.merge':
        'tf.compat.v1.sparse.merge',
    'tf.sparse.placeholder':
        'tf.compat.v1.sparse.placeholder',
    'tf.sparse.reduce_max_sparse':
        'tf.compat.v1.sparse.reduce_max_sparse',
    'tf.sparse.reduce_sum_sparse':
        'tf.compat.v1.sparse.reduce_sum_sparse',
    'tf.sparse_fill_empty_rows':
        'tf.sparse.fill_empty_rows',
    'tf.sparse_mask':
        'tf.sparse.mask',
    'tf.sparse_maximum':
        'tf.sparse.maximum',
    'tf.sparse_merge':
        'tf.compat.v1.sparse_merge',
    'tf.sparse_minimum':
        'tf.sparse.minimum',
    'tf.sparse_placeholder':
        'tf.compat.v1.sparse_placeholder',
    'tf.sparse_reduce_max_sparse':
        'tf.compat.v1.sparse_reduce_max_sparse',
    'tf.sparse_reduce_sum_sparse':
        'tf.compat.v1.sparse_reduce_sum_sparse',
    'tf.sparse_reorder':
        'tf.sparse.reorder',
    'tf.sparse_reset_shape':
        'tf.sparse.reset_shape',
    'tf.sparse_reshape':
        'tf.sparse.reshape',
    'tf.sparse_retain':
        'tf.sparse.retain',
    'tf.sparse_segment_mean':
        'tf.compat.v1.sparse_segment_mean',
    'tf.sparse_segment_sqrt_n':
        'tf.compat.v1.sparse_segment_sqrt_n',
    'tf.sparse_segment_sum':
        'tf.compat.v1.sparse_segment_sum',
    'tf.sparse_slice':
        'tf.sparse.slice',
    'tf.sparse_softmax':
        'tf.sparse.softmax',
    'tf.sparse_tensor_dense_matmul':
        'tf.sparse.sparse_dense_matmul',
    'tf.sparse_tensor_to_dense':
        'tf.sparse.to_dense',
    'tf.sparse_to_dense':
        'tf.compat.v1.sparse_to_dense',
    'tf.sparse_to_indicator':
        'tf.sparse.to_indicator',
    'tf.sparse_transpose':
        'tf.sparse.transpose',
    'tf.spectral.dct':
        'tf.signal.dct',
    'tf.spectral.fft':
        'tf.signal.fft',
    'tf.spectral.fft2d':
        'tf.signal.fft2d',
    'tf.spectral.fft3d':
        'tf.signal.fft3d',
    'tf.spectral.idct':
        'tf.signal.idct',
    'tf.spectral.ifft':
        'tf.signal.ifft',
    'tf.spectral.ifft2d':
        'tf.signal.ifft2d',
    'tf.spectral.ifft3d':
        'tf.signal.ifft3d',
    'tf.spectral.irfft':
        'tf.signal.irfft',
    'tf.spectral.irfft2d':
        'tf.signal.irfft2d',
    'tf.spectral.irfft3d':
        'tf.signal.irfft3d',
    'tf.spectral.rfft':
        'tf.signal.rfft',
    'tf.spectral.rfft2d':
        'tf.signal.rfft2d',
    'tf.spectral.rfft3d':
        'tf.signal.rfft3d',
    'tf.squared_difference':
        'tf.math.squared_difference',
    'tf.string_join':
        'tf.strings.join',
    'tf.string_strip':
        'tf.strings.strip',
    'tf.string_to_hash_bucket_fast':
        'tf.strings.to_hash_bucket_fast',
    'tf.string_to_hash_bucket_strong':
        'tf.strings.to_hash_bucket_strong',
    'tf.summary.Event':
        'tf.compat.v1.summary.Event',
    'tf.summary.FileWriter':
        'tf.compat.v1.summary.FileWriter',
    'tf.summary.FileWriterCache':
        'tf.compat.v1.summary.FileWriterCache',
    'tf.summary.SessionLog':
        'tf.compat.v1.summary.SessionLog',
    'tf.summary.Summary':
        'tf.compat.v1.summary.Summary',
    'tf.summary.SummaryDescription':
        'tf.compat.v1.summary.SummaryDescription',
    'tf.summary.TaggedRunMetadata':
        'tf.compat.v1.summary.TaggedRunMetadata',
    'tf.summary.all_v2_summary_ops':
        'tf.compat.v1.summary.all_v2_summary_ops',
    'tf.summary.audio':
        'tf.compat.v1.summary.audio',
    'tf.summary.get_summary_description':
        'tf.compat.v1.summary.get_summary_description',
    'tf.summary.histogram':
        'tf.compat.v1.summary.histogram',
    'tf.summary.image':
        'tf.compat.v1.summary.image',
    'tf.summary.initialize':
        'tf.compat.v1.summary.initialize',
    'tf.summary.merge':
        'tf.compat.v1.summary.merge',
    'tf.summary.merge_all':
        'tf.compat.v1.summary.merge_all',
    'tf.summary.scalar':
        'tf.compat.v1.summary.scalar',
    'tf.summary.tensor_summary':
        'tf.compat.v1.summary.tensor_summary',
    'tf.summary.text':
        'tf.compat.v1.summary.text',
    'tf.svd':
        'tf.linalg.svd',
    'tf.tables_initializer':
        'tf.compat.v1.tables_initializer',
    'tf.tensor_scatter_add':
        'tf.tensor_scatter_nd_add',
    'tf.tensor_scatter_sub':
        'tf.tensor_scatter_nd_sub',
    'tf.tensor_scatter_update':
        'tf.tensor_scatter_nd_update',
    'tf.test.StubOutForTesting':
        'tf.compat.v1.test.StubOutForTesting',
    'tf.test.compute_gradient_error':
        'tf.compat.v1.test.compute_gradient_error',
    'tf.test.get_temp_dir':
        'tf.compat.v1.test.get_temp_dir',
    'tf.test.mock':
        'tf.compat.v1.test.mock',
    'tf.test.test_src_dir_path':
        'tf.compat.v1.test.test_src_dir_path',
    'tf.to_bfloat16':
        'tf.compat.v1.to_bfloat16',
    'tf.to_complex128':
        'tf.compat.v1.to_complex128',
    'tf.to_complex64':
        'tf.compat.v1.to_complex64',
    'tf.to_double':
        'tf.compat.v1.to_double',
    'tf.to_float':
        'tf.compat.v1.to_float',
    'tf.to_int32':
        'tf.compat.v1.to_int32',
    'tf.to_int64':
        'tf.compat.v1.to_int64',
    'tf.tpu.CrossShardOptimizer':
        'tf.compat.v1.tpu.CrossShardOptimizer',
    'tf.tpu.PaddingSpec':
        'tf.compat.v1.tpu.PaddingSpec',
    'tf.tpu.batch_parallel':
        'tf.compat.v1.tpu.batch_parallel',
    'tf.tpu.bfloat16_scope':
        'tf.compat.v1.tpu.bfloat16_scope',
    'tf.tpu.core':
        'tf.compat.v1.tpu.core',
    'tf.tpu.cross_replica_sum':
        'tf.compat.v1.tpu.cross_replica_sum',
    'tf.tpu.experimental.AdagradParameters':
        'tf.compat.v1.tpu.experimental.AdagradParameters',
    'tf.tpu.experimental.AdamParameters':
        'tf.compat.v1.tpu.experimental.AdamParameters',
    'tf.tpu.experimental.FtrlParameters':
        'tf.compat.v1.tpu.experimental.FtrlParameters',
    'tf.tpu.experimental.StochasticGradientDescentParameters':
        'tf.compat.v1.tpu.experimental.StochasticGradientDescentParameters',
    'tf.tpu.experimental.embedding_column':
        'tf.compat.v1.tpu.experimental.embedding_column',
    'tf.tpu.experimental.shared_embedding_columns':
        'tf.compat.v1.tpu.experimental.shared_embedding_columns',
    'tf.tpu.initialize_system':
        'tf.compat.v1.tpu.initialize_system',
    'tf.tpu.outside_compilation':
        'tf.compat.v1.tpu.outside_compilation',
    'tf.tpu.replicate':
        'tf.compat.v1.tpu.replicate',
    'tf.tpu.rewrite':
        'tf.compat.v1.tpu.rewrite',
    'tf.tpu.shard':
        'tf.compat.v1.tpu.shard',
    'tf.tpu.shutdown_system':
        'tf.compat.v1.tpu.shutdown_system',
    'tf.tpu.XLAOptions':
        'tf.compat.v1.tpu.XLAOptions',
    'tf.trace':
        'tf.linalg.trace',
    'tf.train.AdadeltaOptimizer':
        'tf.compat.v1.train.AdadeltaOptimizer',
    'tf.train.AdagradDAOptimizer':
        'tf.compat.v1.train.AdagradDAOptimizer',
    'tf.train.AdagradOptimizer':
        'tf.compat.v1.train.AdagradOptimizer',
    'tf.train.AdamOptimizer':
        'tf.compat.v1.train.AdamOptimizer',
    'tf.train.CheckpointSaverHook':
        'tf.estimator.CheckpointSaverHook',
    'tf.train.CheckpointSaverListener':
        'tf.estimator.CheckpointSaverListener',
    'tf.train.ChiefSessionCreator':
        'tf.compat.v1.train.ChiefSessionCreator',
    'tf.train.FeedFnHook':
        'tf.estimator.FeedFnHook',
    'tf.train.FinalOpsHook':
        'tf.estimator.FinalOpsHook',
    'tf.train.FtrlOptimizer':
        'tf.compat.v1.train.FtrlOptimizer',
    'tf.train.GlobalStepWaiterHook':
        'tf.estimator.GlobalStepWaiterHook',
    'tf.train.GradientDescentOptimizer':
        'tf.compat.v1.train.GradientDescentOptimizer',
    'tf.train.LoggingTensorHook':
        'tf.estimator.LoggingTensorHook',
    'tf.train.LooperThread':
        'tf.compat.v1.train.LooperThread',
    'tf.train.MomentumOptimizer':
        'tf.compat.v1.train.MomentumOptimizer',
    'tf.train.MonitoredSession':
        'tf.compat.v1.train.MonitoredSession',
    'tf.train.MonitoredTrainingSession':
        'tf.compat.v1.train.MonitoredTrainingSession',
    'tf.train.NanLossDuringTrainingError':
        'tf.estimator.NanLossDuringTrainingError',
    'tf.train.NanTensorHook':
        'tf.estimator.NanTensorHook',
    'tf.train.NewCheckpointReader':
        'tf.compat.v1.train.NewCheckpointReader',
    'tf.train.Optimizer':
        'tf.compat.v1.train.Optimizer',
    'tf.train.ProfilerHook':
        'tf.estimator.ProfilerHook',
    'tf.train.ProximalAdagradOptimizer':
        'tf.compat.v1.train.ProximalAdagradOptimizer',
    'tf.train.ProximalGradientDescentOptimizer':
        'tf.compat.v1.train.ProximalGradientDescentOptimizer',
    'tf.train.QueueRunner':
        'tf.compat.v1.train.QueueRunner',
    'tf.train.RMSPropOptimizer':
        'tf.compat.v1.train.RMSPropOptimizer',
    'tf.train.Saver':
        'tf.compat.v1.train.Saver',
    'tf.train.SaverDef':
        'tf.compat.v1.train.SaverDef',
    'tf.train.Scaffold':
        'tf.compat.v1.train.Scaffold',
    'tf.train.SecondOrStepTimer':
        'tf.estimator.SecondOrStepTimer',
    'tf.train.Server':
        'tf.distribute.Server',
    'tf.train.SessionCreator':
        'tf.compat.v1.train.SessionCreator',
    'tf.train.SessionManager':
        'tf.compat.v1.train.SessionManager',
    'tf.train.SessionRunArgs':
        'tf.estimator.SessionRunArgs',
    'tf.train.SessionRunContext':
        'tf.estimator.SessionRunContext',
    'tf.train.SessionRunHook':
        'tf.estimator.SessionRunHook',
    'tf.train.SessionRunValues':
        'tf.estimator.SessionRunValues',
    'tf.train.SingularMonitoredSession':
        'tf.compat.v1.train.SingularMonitoredSession',
    'tf.train.StepCounterHook':
        'tf.estimator.StepCounterHook',
    'tf.train.StopAtStepHook':
        'tf.estimator.StopAtStepHook',
    'tf.train.SummarySaverHook':
        'tf.estimator.SummarySaverHook',
    'tf.train.Supervisor':
        'tf.compat.v1.train.Supervisor',
    'tf.train.SyncReplicasOptimizer':
        'tf.compat.v1.train.SyncReplicasOptimizer',
    'tf.train.VocabInfo':
        'tf.estimator.VocabInfo',
    'tf.train.WorkerSessionCreator':
        'tf.compat.v1.train.WorkerSessionCreator',
    'tf.train.add_queue_runner':
        'tf.compat.v1.train.add_queue_runner',
    'tf.train.assert_global_step':
        'tf.compat.v1.train.assert_global_step',
    'tf.train.basic_train_loop':
        'tf.compat.v1.train.basic_train_loop',
    'tf.train.batch':
        'tf.compat.v1.train.batch',
    'tf.train.batch_join':
        'tf.compat.v1.train.batch_join',
    'tf.train.checkpoint_exists':
        'tf.compat.v1.train.checkpoint_exists',
    'tf.train.cosine_decay':
        'tf.compat.v1.train.cosine_decay',
    'tf.train.cosine_decay_restarts':
        'tf.compat.v1.train.cosine_decay_restarts',
    'tf.train.create_global_step':
        'tf.compat.v1.train.create_global_step',
    'tf.train.do_quantize_training_on_graphdef':
        'tf.compat.v1.train.do_quantize_training_on_graphdef',
    'tf.train.experimental.DynamicLossScale':
        'tf.compat.v1.train.experimental.DynamicLossScale',
    'tf.train.experimental.FixedLossScale':
        'tf.compat.v1.train.experimental.FixedLossScale',
    'tf.train.experimental.LossScale':
        'tf.compat.v1.train.experimental.LossScale',
    'tf.train.experimental.MixedPrecisionLossScaleOptimizer':
        'tf.compat.v1.train.experimental.MixedPrecisionLossScaleOptimizer',
    'tf.train.experimental.disable_mixed_precision_graph_rewrite':
        'tf.compat.v1.train.experimental.disable_mixed_precision_graph_rewrite',
    'tf.train.experimental.enable_mixed_precision_graph_rewrite':
        'tf.compat.v1.train.experimental.enable_mixed_precision_graph_rewrite',
    'tf.train.exponential_decay':
        'tf.compat.v1.train.exponential_decay',
    'tf.train.export_meta_graph':
        'tf.compat.v1.train.export_meta_graph',
    'tf.train.generate_checkpoint_state_proto':
        'tf.compat.v1.train.generate_checkpoint_state_proto',
    'tf.train.get_checkpoint_mtimes':
        'tf.compat.v1.train.get_checkpoint_mtimes',
    'tf.train.get_global_step':
        'tf.compat.v1.train.get_global_step',
    'tf.train.get_or_create_global_step':
        'tf.compat.v1.train.get_or_create_global_step',
    'tf.train.global_step':
        'tf.compat.v1.train.global_step',
    'tf.train.import_meta_graph':
        'tf.compat.v1.train.import_meta_graph',
    'tf.train.init_from_checkpoint':
        'tf.compat.v1.train.init_from_checkpoint',
    'tf.train.input_producer':
        'tf.compat.v1.train.input_producer',
    'tf.train.inverse_time_decay':
        'tf.compat.v1.train.inverse_time_decay',
    'tf.train.limit_epochs':
        'tf.compat.v1.train.limit_epochs',
    'tf.train.linear_cosine_decay':
        'tf.compat.v1.train.linear_cosine_decay',
    'tf.train.match_filenames_once':
        'tf.io.match_filenames_once',
    'tf.train.maybe_batch':
        'tf.compat.v1.train.maybe_batch',
    'tf.train.maybe_batch_join':
        'tf.compat.v1.train.maybe_batch_join',
    'tf.train.maybe_shuffle_batch':
        'tf.compat.v1.train.maybe_shuffle_batch',
    'tf.train.maybe_shuffle_batch_join':
        'tf.compat.v1.train.maybe_shuffle_batch_join',
    'tf.train.natural_exp_decay':
        'tf.compat.v1.train.natural_exp_decay',
    'tf.train.noisy_linear_cosine_decay':
        'tf.compat.v1.train.noisy_linear_cosine_decay',
    'tf.train.piecewise_constant':
        'tf.compat.v1.train.piecewise_constant',
    'tf.train.piecewise_constant_decay':
        'tf.compat.v1.train.piecewise_constant_decay',
    'tf.train.polynomial_decay':
        'tf.compat.v1.train.polynomial_decay',
    'tf.train.queue_runner.QueueRunner':
        'tf.compat.v1.train.queue_runner.QueueRunner',
    'tf.train.queue_runner.add_queue_runner':
        'tf.compat.v1.train.queue_runner.add_queue_runner',
    'tf.train.queue_runner.start_queue_runners':
        'tf.compat.v1.train.queue_runner.start_queue_runners',
    'tf.train.range_input_producer':
        'tf.compat.v1.train.range_input_producer',
    'tf.train.remove_checkpoint':
        'tf.compat.v1.train.remove_checkpoint',
    'tf.train.replica_device_setter':
        'tf.compat.v1.train.replica_device_setter',
    'tf.train.shuffle_batch':
        'tf.compat.v1.train.shuffle_batch',
    'tf.train.shuffle_batch_join':
        'tf.compat.v1.train.shuffle_batch_join',
    'tf.train.slice_input_producer':
        'tf.compat.v1.train.slice_input_producer',
    'tf.train.start_queue_runners':
        'tf.compat.v1.train.start_queue_runners',
    'tf.train.string_input_producer':
        'tf.compat.v1.train.string_input_producer',
    'tf.train.summary_iterator':
        'tf.compat.v1.train.summary_iterator',
    'tf.train.update_checkpoint_state':
        'tf.compat.v1.train.update_checkpoint_state',
    'tf.train.warm_start':
        'tf.compat.v1.train.warm_start',
    'tf.train.write_graph':
        'tf.io.write_graph',
    'tf.trainable_variables':
        'tf.compat.v1.trainable_variables',
    'tf.truncated_normal':
        'tf.random.truncated_normal',
    'tf.uniform_unit_scaling_initializer':
        'tf.compat.v1.uniform_unit_scaling_initializer',
    'tf.unsorted_segment_max':
        'tf.math.unsorted_segment_max',
    'tf.unsorted_segment_mean':
        'tf.math.unsorted_segment_mean',
    'tf.unsorted_segment_min':
        'tf.math.unsorted_segment_min',
    'tf.unsorted_segment_prod':
        'tf.math.unsorted_segment_prod',
    'tf.unsorted_segment_sqrt_n':
        'tf.math.unsorted_segment_sqrt_n',
    'tf.unsorted_segment_sum':
        'tf.math.unsorted_segment_sum',
    'tf.variable_axis_size_partitioner':
        'tf.compat.v1.variable_axis_size_partitioner',
    'tf.variable_op_scope':
        'tf.compat.v1.variable_op_scope',
    'tf.variable_scope':
        'tf.compat.v1.variable_scope',
    'tf.variables_initializer':
        'tf.compat.v1.variables_initializer',
    'tf.verify_tensor_all_finite':
        'tf.compat.v1.verify_tensor_all_finite',
    'tf.wrap_function':
        'tf.compat.v1.wrap_function',
    'tf.write_file':
        'tf.io.write_file',
    'tf.zeta':
        'tf.math.zeta'
}



# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Operations for writing summary data, for use in analysis and visualization.

See the [Summaries and
TensorBoard](https://www.tensorflow.org/guide/summaries_and_tensorboard) guide.

"""

import sys as _sys

from . import experimental
from tensorflow.python.ops.summary_ops_v2 import SummaryWriter
from tensorflow.python.ops.summary_ops_v2 import create_file_writer_v2 as create_file_writer
from tensorflow.python.ops.summary_ops_v2 import create_noop_writer
from tensorflow.python.ops.summary_ops_v2 import flush
from tensorflow.python.ops.summary_ops_v2 import graph
from tensorflow.python.ops.summary_ops_v2 import record_if
from tensorflow.python.ops.summary_ops_v2 import should_record_summaries
from tensorflow.python.ops.summary_ops_v2 import trace_export
from tensorflow.python.ops.summary_ops_v2 import trace_off
from tensorflow.python.ops.summary_ops_v2 import trace_on
from tensorflow.python.ops.summary_ops_v2 import write


# Copyright 2015 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Utility functions for training."""
from tensorflow.python.eager import context
from tensorflow.python.framework import dtypes
from tensorflow.python.framework import graph_io
from tensorflow.python.framework import ops
from tensorflow.python.ops import init_ops
from tensorflow.python.ops import resource_variable_ops
from tensorflow.python.ops import state_ops
from tensorflow.python.ops import variable_scope
from tensorflow.python.ops import variables
from tensorflow.python.platform import tf_logging as logging
from tensorflow.python.util.tf_export import tf_export

# Picked a long key value to minimize the chance of collision with user defined
# collection keys.
GLOBAL_STEP_READ_KEY = 'global_step_read_op_cache'

# TODO(drpng): remove this after legacy uses are resolved.
write_graph = graph_io.write_graph


@tf_export(v1=['train.global_step'])
def global_step(sess, global_step_tensor):
  """Small helper to get the global step.

  ```python
  # Create a variable to hold the global_step.
  global_step_tensor = tf.Variable(10, trainable=False, name='global_step')
  # Create a session.
  sess = tf.compat.v1.Session()
  # Initialize the variable
  sess.run(global_step_tensor.initializer)
  # Get the variable value.
  print('global_step: %s' % tf.compat.v1.train.global_step(sess,
  global_step_tensor))

  global_step: 10
  ```

  Args:
    sess: A TensorFlow `Session` object.
    global_step_tensor:  `Tensor` or the `name` of the operation that contains
      the global step.

  Returns:
    The global step value.
  """
  if context.executing_eagerly():
    return int(global_step_tensor.numpy())
  return int(sess.run(global_step_tensor))


@tf_export(v1=['train.get_global_step'])
def get_global_step(graph=None):
  """Get the global step tensor.

  The global step tensor must be an integer variable. We first try to find it
  in the collection `GLOBAL_STEP`, or by name `global_step:0`.

  Args:
    graph: The graph to find the global step in. If missing, use default graph.

  Returns:
    The global step variable, or `None` if none was found.

  Raises:
    TypeError: If the global step tensor has a non-integer type, or if it is not
      a `Variable`.

  @compatibility(TF2)
  With the deprecation of global graphs, TF no longer tracks variables in
  collections. In other words, there are no global variables in TF2. Thus, the
  global step functions have been removed  (`get_or_create_global_step`,
  `create_global_step`, `get_global_step`) . You have two options for migrating:

  1. Create a Keras optimizer, which generates an `iterations` variable. This
     variable is automatically incremented when calling `apply_gradients`.
  2. Manually create and increment a `tf.Variable`.

  Below is an example of migrating away from using a global step to using a
  Keras optimizer:

  Define a dummy model and loss:

  >>> def compute_loss(x):
  ...   v = tf.Variable(3.0)
  ...   y = x * v
  ...   loss = x * 5 - x * v
  ...   return loss, [v]

  Before migrating:

  >>> g = tf.Graph()
  >>> with g.as_default():
  ...   x = tf.compat.v1.placeholder(tf.float32, [])
  ...   loss, var_list = compute_loss(x)
  ...   global_step = tf.compat.v1.train.get_or_create_global_step()
  ...   global_init = tf.compat.v1.global_variables_initializer()
  ...   optimizer = tf.compat.v1.train.GradientDescentOptimizer(0.1)
  ...   train_op = optimizer.minimize(loss, global_step, var_list)
  >>> sess = tf.compat.v1.Session(graph=g)
  >>> sess.run(global_init)
  >>> print("before training:", sess.run(global_step))
  before training: 0
  >>> sess.run(train_op, feed_dict={x: 3})
  >>> print("after training:", sess.run(global_step))
  after training: 1

  Using `get_global_step`:

  >>> with g.as_default():
  ...   print(sess.run(tf.compat.v1.train.get_global_step()))
  1

  Migrating to a Keras optimizer:

  >>> optimizer = tf.keras.optimizers.SGD(.01)
  >>> print("before training:", optimizer.iterations.numpy())
  before training: 0
  >>> with tf.GradientTape() as tape:
  ...   loss, var_list = compute_loss(3)
  ...   grads = tape.gradient(loss, var_list)
  ...   optimizer.apply_gradients(zip(grads, var_list))
  >>> print("after training:", optimizer.iterations.numpy())
  after training: 1

  @end_compatibility
  """
  graph = graph or ops.get_default_graph()
  global_step_tensor = None
  global_step_tensors = graph.get_collection(ops.GraphKeys.GLOBAL_STEP)
  if len(global_step_tensors) == 1:
    global_step_tensor = global_step_tensors[0]
  elif not global_step_tensors:
    try:
      global_step_tensor = graph.get_tensor_by_name('global_step:0')
    except KeyError:
      return None
  else:
    logging.error('Multiple tensors in global_step collection.')
    return None

  assert_global_step(global_step_tensor)
  return global_step_tensor


@tf_export(v1=['train.create_global_step'])
def create_global_step(graph=None):
  """Create global step tensor in graph.

  Args:
    graph: The graph in which to create the global step tensor. If missing, use
      default graph.

  Returns:
    Global step tensor.

  Raises:
    ValueError: if global step tensor is already defined.

  @compatibility(TF2)
  With the deprecation of global graphs, TF no longer tracks variables in
  collections. In other words, there are no global variables in TF2. Thus, the
  global step functions have been removed  (`get_or_create_global_step`,
  `create_global_step`, `get_global_step`) . You have two options for migrating:

  1. Create a Keras optimizer, which generates an `iterations` variable. This
     variable is automatically incremented when calling `apply_gradients`.
  2. Manually create and increment a `tf.Variable`.

  Below is an example of migrating away from using a global step to using a
  Keras optimizer:

  Define a dummy model and loss:

  >>> def compute_loss(x):
  ...   v = tf.Variable(3.0)
  ...   y = x * v
  ...   loss = x * 5 - x * v
  ...   return loss, [v]

  Before migrating:

  >>> g = tf.Graph()
  >>> with g.as_default():
  ...   x = tf.compat.v1.placeholder(tf.float32, [])
  ...   loss, var_list = compute_loss(x)
  ...   global_step = tf.compat.v1.train.create_global_step()
  ...   global_init = tf.compat.v1.global_variables_initializer()
  ...   optimizer = tf.compat.v1.train.GradientDescentOptimizer(0.1)
  ...   train_op = optimizer.minimize(loss, global_step, var_list)
  >>> sess = tf.compat.v1.Session(graph=g)
  >>> sess.run(global_init)
  >>> print("before training:", sess.run(global_step))
  before training: 0
  >>> sess.run(train_op, feed_dict={x: 3})
  >>> print("after training:", sess.run(global_step))
  after training: 1

  Migrating to a Keras optimizer:

  >>> optimizer = tf.keras.optimizers.SGD(.01)
  >>> print("before training:", optimizer.iterations.numpy())
  before training: 0
  >>> with tf.GradientTape() as tape:
  ...   loss, var_list = compute_loss(3)
  ...   grads = tape.gradient(loss, var_list)
  ...   optimizer.apply_gradients(zip(grads, var_list))
  >>> print("after training:", optimizer.iterations.numpy())
  after training: 1

  @end_compatibility
  """
  graph = graph or ops.get_default_graph()
  if get_global_step(graph) is not None:
    raise ValueError('"global_step" already exists.')
  if context.executing_eagerly():
    with ops.device('cpu:0'):
      return variable_scope.get_variable(
          ops.GraphKeys.GLOBAL_STEP,
          shape=[],
          dtype=dtypes.int64,
          initializer=init_ops.zeros_initializer(),
          trainable=False,
          aggregation=variables.VariableAggregation.ONLY_FIRST_REPLICA,
          collections=[
              ops.GraphKeys.GLOBAL_VARIABLES, ops.GraphKeys.GLOBAL_STEP
          ])
  # Create in proper graph and base name_scope.
  with graph.as_default() as g, g.name_scope(None):
    return variable_scope.get_variable(
        ops.GraphKeys.GLOBAL_STEP,
        shape=[],
        dtype=dtypes.int64,
        initializer=init_ops.zeros_initializer(),
        trainable=False,
        aggregation=variables.VariableAggregation.ONLY_FIRST_REPLICA,
        collections=[ops.GraphKeys.GLOBAL_VARIABLES, ops.GraphKeys.GLOBAL_STEP])


@tf_export(v1=['train.get_or_create_global_step'])
def get_or_create_global_step(graph=None):
  """Returns and create (if necessary) the global step tensor.

  Args:
    graph: The graph in which to create the global step tensor. If missing, use
      default graph.

  Returns:
    The global step tensor.

  @compatibility(TF2)
  With the deprecation of global graphs, TF no longer tracks variables in
  collections. In other words, there are no global variables in TF2. Thus, the
  global step functions have been removed  (`get_or_create_global_step`,
  `create_global_step`, `get_global_step`) . You have two options for migrating:

  1. Create a Keras optimizer, which generates an `iterations` variable. This
     variable is automatically incremented when calling `apply_gradients`.
  2. Manually create and increment a `tf.Variable`.

  Below is an example of migrating away from using a global step to using a
  Keras optimizer:

  Define a dummy model and loss:

  >>> def compute_loss(x):
  ...   v = tf.Variable(3.0)
  ...   y = x * v
  ...   loss = x * 5 - x * v
  ...   return loss, [v]

  Before migrating:

  >>> g = tf.Graph()
  >>> with g.as_default():
  ...   x = tf.compat.v1.placeholder(tf.float32, [])
  ...   loss, var_list = compute_loss(x)
  ...   global_step = tf.compat.v1.train.get_or_create_global_step()
  ...   global_init = tf.compat.v1.global_variables_initializer()
  ...   optimizer = tf.compat.v1.train.GradientDescentOptimizer(0.1)
  ...   train_op = optimizer.minimize(loss, global_step, var_list)
  >>> sess = tf.compat.v1.Session(graph=g)
  >>> sess.run(global_init)
  >>> print("before training:", sess.run(global_step))
  before training: 0
  >>> sess.run(train_op, feed_dict={x: 3})
  >>> print("after training:", sess.run(global_step))
  after training: 1

  Migrating to a Keras optimizer:

  >>> optimizer = tf.keras.optimizers.SGD(.01)
  >>> print("before training:", optimizer.iterations.numpy())
  before training: 0
  >>> with tf.GradientTape() as tape:
  ...   loss, var_list = compute_loss(3)
  ...   grads = tape.gradient(loss, var_list)
  ...   optimizer.apply_gradients(zip(grads, var_list))
  >>> print("after training:", optimizer.iterations.numpy())
  after training: 1

  @end_compatibility
  """
  graph = graph or ops.get_default_graph()
  global_step_tensor = get_global_step(graph)
  if global_step_tensor is None:
    global_step_tensor = create_global_step(graph)
  return global_step_tensor


@tf_export(v1=['train.assert_global_step'])
def assert_global_step(global_step_tensor):
  """Asserts `global_step_tensor` is a scalar int `Variable` or `Tensor`.

  Args:
    global_step_tensor: `Tensor` to test.
  """
  if not (isinstance(global_step_tensor, variables.Variable) or
          isinstance(global_step_tensor, ops.Tensor) or
          resource_variable_ops.is_resource_variable(global_step_tensor)):
    raise TypeError('Existing "global_step" must be a Variable or Tensor: %s.' %
                    global_step_tensor)

  if not global_step_tensor.dtype.base_dtype.is_integer:
    raise TypeError('Existing "global_step" does not have integer type: %s' %
                    global_step_tensor.dtype)

  if (global_step_tensor.get_shape().ndims != 0 and
      global_step_tensor.get_shape().is_fully_defined()):
    raise TypeError('Existing "global_step" is not scalar: %s' %
                    global_step_tensor.get_shape())


def _get_global_step_read(graph=None):
  """Gets global step read tensor in graph.

  Args:
    graph: The graph in which to create the global step read tensor. If missing,
      use default graph.

  Returns:
    Global step read tensor.

  Raises:
    RuntimeError: if multiple items found in collection GLOBAL_STEP_READ_KEY.
  """
  graph = graph or ops.get_default_graph()
  global_step_read_tensors = graph.get_collection(GLOBAL_STEP_READ_KEY)
  if len(global_step_read_tensors) > 1:
    raise RuntimeError('There are multiple items in collection {}. '
                       'There should be only one.'.format(GLOBAL_STEP_READ_KEY))

  if len(global_step_read_tensors) == 1:
    return global_step_read_tensors[0]
  return None


def _get_or_create_global_step_read(graph=None):
  """Gets or creates global step read tensor in graph.

  Args:
    graph: The graph in which to create the global step read tensor. If missing,
      use default graph.

  Returns:
    Global step read tensor if there is global_step_tensor else return None.
  """
  graph = graph or ops.get_default_graph()
  global_step_read_tensor = _get_global_step_read(graph)
  if global_step_read_tensor is not None:
    return global_step_read_tensor
  global_step_tensor = get_global_step(graph)
  if global_step_tensor is None:
    return None
  # add 'zero' so that it will create a copy of variable as Tensor.
  with graph.as_default() as g, g.name_scope(None):
    with g.name_scope(global_step_tensor.op.name + '/'):
      # using initialized_value to ensure that global_step is initialized before
      # this run. This is needed for example Estimator makes all model_fn build
      # under global_step_read_tensor dependency.
      global_step_value = global_step_tensor.initialized_value() if isinstance(
          global_step_tensor, variables.Variable) else global_step_tensor
      global_step_read_tensor = global_step_value + 0
      ops.add_to_collection(GLOBAL_STEP_READ_KEY, global_step_read_tensor)
  return _get_global_step_read(graph)


def _increment_global_step(increment, graph=None):
  graph = graph or ops.get_default_graph()
  global_step_tensor = get_global_step(graph)
  if global_step_tensor is None:
    raise ValueError(
        'Global step tensor should be created by '
        'tf.train.get_or_create_global_step before calling increment.')
  global_step_read_tensor = _get_or_create_global_step_read(graph)
  with graph.as_default() as g, g.name_scope(None):
    with g.name_scope(global_step_tensor.op.name + '/'):
      with ops.control_dependencies([global_step_read_tensor]):
        return state_ops.assign_add(global_step_tensor, increment)



# Copyright 2016 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Command parsing module for TensorFlow Debugger (tfdbg)."""
import argparse
import ast
import re
import sys


_BRACKETS_PATTERN = re.compile(r"\[[^\]]*\]")
_QUOTES_PATTERN = re.compile(r"(\"[^\"]*\"|\'[^\']*\')")
_WHITESPACE_PATTERN = re.compile(r"\s+")

_NUMBER_PATTERN = re.compile(r"[-+]?(\d+(\.\d*)?|\.\d+)([eE][-+]?\d+)?")


class Interval(object):
  """Represents an interval between a start and end value."""

  def __init__(self, start, start_included, end, end_included):
    self.start = start
    self.start_included = start_included
    self.end = end
    self.end_included = end_included

  def contains(self, value):
    if value < self.start or value == self.start and not self.start_included:
      return False
    if value > self.end or value == self.end and not self.end_included:
      return False
    return True

  def __eq__(self, other):
    return (self.start == other.start and
            self.start_included == other.start_included and
            self.end == other.end and
            self.end_included == other.end_included)


def parse_command(command):
  """Parse command string into a list of arguments.

  - Disregards whitespace inside double quotes and brackets.
  - Strips paired leading and trailing double quotes in arguments.
  - Splits the command at whitespace.

  Nested double quotes and brackets are not handled.

  Args:
    command: (str) Input command.

  Returns:
    (list of str) List of arguments.
  """

  command = command.strip()
  if not command:
    return []

  brackets_intervals = [f.span() for f in _BRACKETS_PATTERN.finditer(command)]
  quotes_intervals = [f.span() for f in _QUOTES_PATTERN.finditer(command)]
  whitespaces_intervals = [
      f.span() for f in _WHITESPACE_PATTERN.finditer(command)
  ]

  if not whitespaces_intervals:
    return [command]

  arguments = []
  idx0 = 0
  for start, end in whitespaces_intervals + [(len(command), None)]:
    # Skip whitespace stretches enclosed in brackets or double quotes.

    if not any(interval[0] < start < interval[1]
               for interval in brackets_intervals + quotes_intervals):
      argument = command[idx0:start]

      # Strip leading and trailing double quote if they are paired.
      if (argument.startswith("\"") and argument.endswith("\"") or
          argument.startswith("'") and argument.endswith("'")):
        argument = argument[1:-1]
      arguments.append(argument)
      idx0 = end

  return arguments


def extract_output_file_path(args):
  """Extract output file path from command arguments.

  Args:
    args: (list of str) command arguments.

  Returns:
    (list of str) Command arguments with the output file path part stripped.
    (str or None) Output file path (if any).

  Raises:
    SyntaxError: If there is no file path after the last ">" character.
  """

  if args and args[-1].endswith(">"):
    raise SyntaxError("Redirect file path is empty")
  elif args and args[-1].startswith(">"):
    try:
      _parse_interval(args[-1])
      if len(args) > 1 and args[-2].startswith("-"):
        output_file_path = None
      else:
        output_file_path = args[-1][1:]
        args = args[:-1]
    except ValueError:
      output_file_path = args[-1][1:]
      args = args[:-1]
  elif len(args) > 1 and args[-2] == ">":
    output_file_path = args[-1]
    args = args[:-2]
  elif args and args[-1].count(">") == 1:
    gt_index = args[-1].index(">")
    if gt_index > 0 and args[-1][gt_index - 1] == "=":
      output_file_path = None
    else:
      output_file_path = args[-1][gt_index + 1:]
      args[-1] = args[-1][:gt_index]
  elif len(args) > 1 and args[-2].endswith(">"):
    output_file_path = args[-1]
    args = args[:-1]
    args[-1] = args[-1][:-1]
  else:
    output_file_path = None

  return args, output_file_path


def parse_tensor_name_with_slicing(in_str):
  """Parse tensor name, potentially suffixed by slicing string.

  Args:
    in_str: (str) Input name of the tensor, potentially followed by a slicing
      string. E.g.: Without slicing string: "hidden/weights/Variable:0", with
      slicing string: "hidden/weights/Variable:0[1, :]"

  Returns:
    (str) name of the tensor
    (str) slicing string, if any. If no slicing string is present, return "".
  """

  if in_str.count("[") == 1 and in_str.endswith("]"):
    tensor_name = in_str[:in_str.index("[")]
    tensor_slicing = in_str[in_str.index("["):]
  else:
    tensor_name = in_str
    tensor_slicing = ""

  return tensor_name, tensor_slicing


def validate_slicing_string(slicing_string):
  """Validate a slicing string.

  Check if the input string contains only brackets, digits, commas and
  colons that are valid characters in numpy-style array slicing.

  Args:
    slicing_string: (str) Input slicing string to be validated.

  Returns:
    (bool) True if and only if the slicing string is valid.
  """

  return bool(re.search(r"^\[(\d|,|\s|:)+\]$", slicing_string))


def _parse_slices(slicing_string):
  """Construct a tuple of slices from the slicing string.

  The string must be a valid slicing string.

  Args:
    slicing_string: (str) Input slicing string to be parsed.

  Returns:
    tuple(slice1, slice2, ...)

  Raises:
    ValueError: If tensor_slicing is not a valid numpy ndarray slicing str.
  """
  parsed = []
  for slice_string in slicing_string[1:-1].split(","):
    indices = slice_string.split(":")
    if len(indices) == 1:
      parsed.append(int(indices[0].strip()))
    elif 2 <= len(indices) <= 3:
      parsed.append(
          slice(*[
              int(index.strip()) if index.strip() else None for index in indices
          ]))
    else:
      raise ValueError("Invalid tensor-slicing string.")
  return tuple(parsed)


def parse_indices(indices_string):
  """Parse a string representing indices.

  For example, if the input is "[1, 2, 3]", the return value will be a list of
  indices: [1, 2, 3]

  Args:
    indices_string: (str) a string representing indices. Can optionally be
      surrounded by a pair of brackets.

  Returns:
    (list of int): Parsed indices.
  """

  # Strip whitespace.
  indices_string = re.sub(r"\s+", "", indices_string)

  # Strip any brackets at the two ends.
  if indices_string.startswith("[") and indices_string.endswith("]"):
    indices_string = indices_string[1:-1]

  return [int(element) for element in indices_string.split(",")]


def parse_ranges(range_string):
  """Parse a string representing numerical range(s).

  Args:
    range_string: (str) A string representing a numerical range or a list of
      them. For example:
        "[-1.0,1.0]", "[-inf, 0]", "[[-inf, -1.0], [1.0, inf]]"

  Returns:
    (list of list of float) A list of numerical ranges parsed from the input
      string.

  Raises:
    ValueError: If the input doesn't represent a range or a list of ranges.
  """

  range_string = range_string.strip()
  if not range_string:
    return []

  if "inf" in range_string:
    range_string = re.sub(r"inf", repr(sys.float_info.max), range_string)

  ranges = ast.literal_eval(range_string)
  if isinstance(ranges, list) and not isinstance(ranges[0], list):
    ranges = [ranges]

  # Verify that ranges is a list of list of numbers.
  for item in ranges:
    if len(item) != 2:
      raise ValueError("Incorrect number of elements in range")
    elif not isinstance(item[0], (int, float)):
      raise ValueError("Incorrect type in the 1st element of range: %s" %
                       type(item[0]))
    elif not isinstance(item[1], (int, float)):
      raise ValueError("Incorrect type in the 2nd element of range: %s" %
                       type(item[0]))

  return ranges


def parse_memory_interval(interval_str):
  """Convert a human-readable memory interval to a tuple of start and end value.

  Args:
    interval_str: (`str`) A human-readable str representing an interval
      (e.g., "[10kB, 20kB]", "<100M", ">100G"). Only the units "kB", "MB", "GB"
      are supported. The "B character at the end of the input `str` may be
      omitted.

  Returns:
    `Interval` object where start and end are in bytes.

  Raises:
    ValueError: if the input is not valid.
  """
  str_interval = _parse_interval(interval_str)
  interval_start = 0
  interval_end = float("inf")
  if str_interval.start:
    interval_start = parse_readable_size_str(str_interval.start)
  if str_interval.end:
    interval_end = parse_readable_size_str(str_interval.end)
  if interval_start > interval_end:
    raise ValueError(
        "Invalid interval %s. Start of interval must be less than or equal "
        "to end of interval." % interval_str)
  return Interval(interval_start, str_interval.start_included,
                  interval_end, str_interval.end_included)


def parse_time_interval(interval_str):
  """Convert a human-readable time interval to a tuple of start and end value.

  Args:
    interval_str: (`str`) A human-readable str representing an interval
      (e.g., "[10us, 20us]", "<100s", ">100ms"). Supported time suffixes are
      us, ms, s.

  Returns:
    `Interval` object where start and end are in microseconds.

  Raises:
    ValueError: if the input is not valid.
  """
  str_interval = _parse_interval(interval_str)
  interval_start = 0
  interval_end = float("inf")
  if str_interval.start:
    interval_start = parse_readable_time_str(str_interval.start)
  if str_interval.end:
    interval_end = parse_readable_time_str(str_interval.end)
  if interval_start > interval_end:
    raise ValueError(
        "Invalid interval %s. Start must be before end of interval." %
        interval_str)
  return Interval(interval_start, str_interval.start_included,
                  interval_end, str_interval.end_included)


def _parse_interval(interval_str):
  """Convert a human-readable interval to a tuple of start and end value.

  Args:
    interval_str: (`str`) A human-readable str representing an interval
      (e.g., "[1M, 2M]", "<100k", ">100ms"). The items following the ">", "<",
      ">=" and "<=" signs have to start with a number (e.g., 3.0, -2, .98).
      The same requirement applies to the items in the parentheses or brackets.

  Returns:
    Interval object where start or end can be None
    if the range is specified as "<N" or ">N" respectively.

  Raises:
    ValueError: if the input is not valid.
  """
  interval_str = interval_str.strip()
  if interval_str.startswith("<="):
    if _NUMBER_PATTERN.match(interval_str[2:].strip()):
      return Interval(start=None, start_included=False,
                      end=interval_str[2:].strip(), end_included=True)
    else:
      raise ValueError("Invalid value string after <= in '%s'" % interval_str)
  if interval_str.startswith("<"):
    if _NUMBER_PATTERN.match(interval_str[1:].strip()):
      return Interval(start=None, start_included=False,
                      end=interval_str[1:].strip(), end_included=False)
    else:
      raise ValueError("Invalid value string after < in '%s'" % interval_str)
  if interval_str.startswith(">="):
    if _NUMBER_PATTERN.match(interval_str[2:].strip()):
      return Interval(start=interval_str[2:].strip(), start_included=True,
                      end=None, end_included=False)
    else:
      raise ValueError("Invalid value string after >= in '%s'" % interval_str)
  if interval_str.startswith(">"):
    if _NUMBER_PATTERN.match(interval_str[1:].strip()):
      return Interval(start=interval_str[1:].strip(), start_included=False,
                      end=None, end_included=False)
    else:
      raise ValueError("Invalid value string after > in '%s'" % interval_str)

  if (not interval_str.startswith(("[", "("))
      or not interval_str.endswith(("]", ")"))):
    raise ValueError(
        "Invalid interval format: %s. Valid formats are: [min, max], "
        "(min, max), <max, >min" % interval_str)
  interval = interval_str[1:-1].split(",")
  if len(interval) != 2:
    raise ValueError(
        "Incorrect interval format: %s. Interval should specify two values: "
        "[min, max] or (min, max)." % interval_str)

  start_item = interval[0].strip()
  if not _NUMBER_PATTERN.match(start_item):
    raise ValueError("Invalid first item in interval: '%s'" % start_item)
  end_item = interval[1].strip()
  if not _NUMBER_PATTERN.match(end_item):
    raise ValueError("Invalid second item in interval: '%s'" % end_item)

  return Interval(start=start_item,
                  start_included=(interval_str[0] == "["),
                  end=end_item,
                  end_included=(interval_str[-1] == "]"))


def parse_readable_size_str(size_str):
  """Convert a human-readable str representation to number of bytes.

  Only the units "kB", "MB", "GB" are supported. The "B character at the end
  of the input `str` may be omitted.

  Args:
    size_str: (`str`) A human-readable str representing a number of bytes
      (e.g., "0", "1023", "1.1kB", "24 MB", "23GB", "100 G".

  Returns:
    (`int`) The parsed number of bytes.

  Raises:
    ValueError: on failure to parse the input `size_str`.
  """

  size_str = size_str.strip()
  if size_str.endswith("B"):
    size_str = size_str[:-1]

  if size_str.isdigit():
    return int(size_str)
  elif size_str.endswith("k"):
    return int(float(size_str[:-1]) * 1024)
  elif size_str.endswith("M"):
    return int(float(size_str[:-1]) * 1048576)
  elif size_str.endswith("G"):
    return int(float(size_str[:-1]) * 1073741824)
  else:
    raise ValueError("Failed to parsed human-readable byte size str: \"%s\"" %
                     size_str)


def parse_readable_time_str(time_str):
  """Parses a time string in the format N, Nus, Nms, Ns.

  Args:
    time_str: (`str`) string consisting of an integer time value optionally
      followed by 'us', 'ms', or 's' suffix. If suffix is not specified,
      value is assumed to be in microseconds. (e.g. 100us, 8ms, 5s, 100).

  Returns:
    Microseconds value.
  """
  def parse_positive_float(value_str):
    value = float(value_str)
    if value < 0:
      raise ValueError(
          "Invalid time %s. Time value must be positive." % value_str)
    return value

  time_str = time_str.strip()
  if time_str.endswith("us"):
    return int(parse_positive_float(time_str[:-2]))
  elif time_str.endswith("ms"):
    return int(parse_positive_float(time_str[:-2]) * 1e3)
  elif time_str.endswith("s"):
    return int(parse_positive_float(time_str[:-1]) * 1e6)
  return int(parse_positive_float(time_str))


def evaluate_tensor_slice(tensor, tensor_slicing):
  """Call eval on the slicing of a tensor, with validation.

  Args:
    tensor: (numpy ndarray) The tensor value.
    tensor_slicing: (str or None) Slicing of the tensor, e.g., "[:, 1]". If
      None, no slicing will be performed on the tensor.

  Returns:
    (numpy ndarray) The sliced tensor.

  Raises:
    ValueError: If tensor_slicing is not a valid numpy ndarray slicing str.
  """

  _ = tensor

  if not validate_slicing_string(tensor_slicing):
    raise ValueError("Invalid tensor-slicing string.")

  return tensor[_parse_slices(tensor_slicing)]


def get_print_tensor_argparser(description):
  """Get an ArgumentParser for a command that prints tensor values.

  Examples of such commands include print_tensor and print_feed.

  Args:
    description: Description of the ArgumentParser.

  Returns:
    An instance of argparse.ArgumentParser.
  """

  ap = argparse.ArgumentParser(
      description=description, usage=argparse.SUPPRESS)
  ap.add_argument(
      "tensor_name",
      type=str,
      help="Name of the tensor, followed by any slicing indices, "
      "e.g., hidden1/Wx_plus_b/MatMul:0, "
      "hidden1/Wx_plus_b/MatMul:0[1, :]")
  ap.add_argument(
      "-n",
      "--number",
      dest="number",
      type=int,
      default=-1,
      help="0-based dump number for the specified tensor. "
      "Required for tensor with multiple dumps.")
  ap.add_argument(
      "-r",
      "--ranges",
      dest="ranges",
      type=str,
      default="",
      help="Numerical ranges to highlight tensor elements in. "
      "Examples: -r 0,1e-8, -r [-0.1,0.1], "
      "-r \"[[-inf, -0.1], [0.1, inf]]\"")
  ap.add_argument(
      "-a",
      "--all",
      dest="print_all",
      action="store_true",
      help="Print the tensor in its entirety, i.e., do not use ellipses.")
  ap.add_argument(
      "-s",
      "--numeric_summary",
      action="store_true",
      help="Include summary for non-empty tensors of numeric (int*, float*, "
      "complex*) and Boolean types.")
  ap.add_argument(
      "-w",
      "--write_path",
      type=str,
      default="",
      help="Path of the numpy file to write the tensor data to, using "
      "numpy.save().")
  return ap



"""Manages a graph of Trackable objects."""
# Copyright 2017 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
import collections
import copy
import weakref

from tensorflow.core.protobuf import trackable_object_graph_pb2
from tensorflow.python.framework import constant_op
from tensorflow.python.framework import dtypes
from tensorflow.python.framework import ops
from tensorflow.python.saved_model import registration
from tensorflow.python.training import optimizer as optimizer_v1
from tensorflow.python.training.saving import saveable_object as saveable_object_lib
from tensorflow.python.training.saving import saveable_object_util
from tensorflow.python.training.tracking import base
from tensorflow.python.training.tracking import trackable_utils
from tensorflow.python.util import object_identity
from tensorflow.python.util.tf_export import tf_export


# Factory and related info used to build a SaveableObject that saves a Trackable
# to checkpoint.
_CheckpointFactoryData = collections.namedtuple(
    "_CheckpointFactoryData", ["factory", "name", "checkpoint_key"])


def _serialize_slot_variables(trackable_objects, node_ids, object_names):
  """Gather and name slot variables."""
  non_slot_objects = list(trackable_objects)
  slot_variables = object_identity.ObjectIdentityDictionary()
  for trackable in non_slot_objects:
    if (isinstance(trackable, optimizer_v1.Optimizer)
        # TODO(b/110718070): Fix Keras imports.
        # Note: dir() is used rather than hasattr() here to avoid triggering
        # custom __getattr__ code, see b/152031870 for context.
        or "_create_or_restore_slot_variable" in dir(trackable)):
      slot_names = trackable.get_slot_names()
      for slot_name in slot_names:
        for original_variable_node_id, original_variable in enumerate(
            non_slot_objects):
          try:
            slot_variable = trackable.get_slot(
                original_variable, slot_name)
          except (AttributeError, KeyError):
            slot_variable = None
          if slot_variable is None:
            continue
          slot_variable._maybe_initialize_trackable()  # pylint: disable=protected-access
          if slot_variable._trackable_children():  # pylint: disable=protected-access
            # TODO(allenl): Gather dependencies of slot variables.
            raise NotImplementedError(
                "Currently only variables with no dependencies can be saved as "
                "slot variables. File a feature request if this limitation "
                "bothers you.")
          if slot_variable in node_ids:
            raise NotImplementedError(
                "A slot variable was re-used as a dependency of a Trackable "
                f"object: {slot_variable}. This is not currently allowed. "
                "File a feature request if this limitation bothers you.")
          checkpoint_name = trackable_utils.slot_variable_key(
              variable_path=object_names[original_variable],
              optimizer_path=object_names[trackable],
              slot_name=slot_name)
          object_names[slot_variable] = checkpoint_name
          slot_variable_node_id = len(trackable_objects)
          node_ids[slot_variable] = slot_variable_node_id
          trackable_objects.append(slot_variable)
          slot_variable_proto = (
              trackable_object_graph_pb2.TrackableObjectGraph
              .TrackableObject.SlotVariableReference(
                  slot_name=slot_name,
                  original_variable_node_id=original_variable_node_id,
                  slot_variable_node_id=slot_variable_node_id))
          slot_variables.setdefault(trackable, []).append(
              slot_variable_proto)
  return slot_variables


def _get_mapped_trackable(trackable, object_map):
  """Returns the mapped trackable if possible, otherwise returns trackable."""
  if object_map is None:
    return trackable
  else:
    return object_map.get(trackable, trackable)


def get_checkpoint_factories_and_keys(object_names, object_map=None):
  """Gets a map of saveable factories and corresponding checkpoint keys.

  Args:
    object_names: a dictionary that maps `Trackable` objects to auto-generated
      string names.
    object_map: a dictionary mapping `Trackable` to copied `Trackable` objects.
      The copied objects are generated from `Trackable._map_resources()` which
      copies the object into another graph. Generally only resource objects
      (e.g. Variables, Tables) will be in this map.
  Returns:
    A tuple of (
      Dictionary mapping trackable -> list of _CheckpointFactoryData,
      Dictionary mapping registered saver name -> {object name -> trackable})
  """
  checkpoint_factory_map = object_identity.ObjectIdentityDictionary()
  registered_savers = collections.defaultdict(dict)
  for trackable, object_name in object_names.items():
    # object_to_save is only used to retrieve the saving functionality. For keys
    # and other data, use the original `trackable`.
    object_to_save = _get_mapped_trackable(trackable, object_map)

    saver_name = registration.get_registered_saver_name(object_to_save)
    if saver_name:
      registered_savers[saver_name][object_name] = trackable
    else:
      checkpoint_factory_map[trackable] = []
      for name, saveable_factory in (
          saveable_object_util.saveable_objects_from_trackable(object_to_save)
          .items()):  # pylint: disable=protected-access
        checkpoint_key = trackable_utils.checkpoint_key(object_name, name)
        checkpoint_factory_map[trackable].append(_CheckpointFactoryData(
            factory=saveable_factory,
            name=name,
            checkpoint_key=checkpoint_key))
  return checkpoint_factory_map, registered_savers


def _add_attributes_to_object_graph_for_registered_savers(
    registered_savers, object_graph_proto, node_ids):
  """Fills the object graph proto with data about the registered savers."""
  for saver_name, trackables in registered_savers.items():
    for object_name, trackable in trackables.items():
      object_proto = object_graph_proto.nodes[node_ids[trackable]]
      object_proto.registered_saver.name = saver_name
      object_proto.registered_saver.object_name = object_name


@tf_export("__internal__.tracking.ObjectGraphView", v1=[])
class ObjectGraphView(object):
  """Gathers and serializes an object graph."""

  def __init__(self, root, saveables_cache=None, attached_dependencies=None):
    """Configure the graph view.

    Args:
      root: A `Trackable` object whose variables (including the variables
        of dependencies, recursively) should be saved. May be a weak reference.
      saveables_cache: A dictionary mapping `Trackable` objects ->
        attribute names -> SaveableObjects, used to avoid re-creating
        SaveableObjects when graph building.
      attached_dependencies: List of dependencies to attach to the root object.
        Used when saving a Checkpoint with a defined root object. To avoid
        reference cycles, this should use the WeakTrackableReference class.
    """
    # ObjectGraphView should never contain a strong reference to root, since it
    # may result in a cycle:
    #   root -> deferred dependencies -> CheckpointPosition
    #   -> CheckpointRestoreCoordinator -> ObjectGraphView -> root
    self._root_ref = (root if isinstance(root, weakref.ref)
                      else weakref.ref(root))
    self._saveables_cache = saveables_cache
    self._attached_dependencies = attached_dependencies

  def __deepcopy__(self, memo):
    # By default, weak references are not copied, which leads to surprising
    # deepcopy behavior. To fix, we first we copy the object itself, then we
    # make a weak reference to the copy.
    strong_root = self._root_ref()
    if strong_root is not None:
      strong_copy = copy.deepcopy(strong_root, memo)
      memo[id(self._root_ref)] = weakref.ref(strong_copy)
    # super() does not have a __deepcopy__, so we need to re-implement it
    copied = super().__new__(type(self))
    memo[id(self)] = copied
    for key, value in vars(self).items():
      setattr(copied, key, copy.deepcopy(value, memo))
    return copied

  def list_children(self, obj, save_type=base.SaveType.CHECKPOINT, **kwargs):
    """Returns all child trackables attached to obj.

    Args:
      obj: A `Trackable` object.
      save_type: A string, can be 'savedmodel' or 'checkpoint'.
      **kwargs: kwargs to use when retrieving the object's children.

    Returns:
      List of all children attached to the object.
    """
    # pylint: disable=protected-access
    obj._maybe_initialize_trackable()
    children = [base.TrackableReference(name, ref) for name, ref
                in obj._trackable_children(save_type, **kwargs).items()]
    # pylint: enable=protected-access

    # GraphView objects may define children of the root object that are not
    # actually attached, e.g. a Checkpoint object's save_counter.
    if obj is self.root and self._attached_dependencies:
      children.extend(self._attached_dependencies)
    return children

  @property
  def saveables_cache(self):
    """Maps Trackable objects -> attribute names -> list(SaveableObjects).

    Used to avoid re-creating SaveableObjects when graph building. None when
    executing eagerly.

    Returns:
      The cache (an object-identity dictionary), or None if caching is disabled.
    """
    return self._saveables_cache

  @property
  def attached_dependencies(self):
    """Returns list of dependencies that should be saved in the checkpoint.

    These dependencies are not tracked by root, but are in the checkpoint.
    This is defined when the user creates a Checkpoint with both root and kwargs
    set.

    Returns:
      A list of TrackableReferences.
    """
    return self._attached_dependencies

  @property
  def root(self):
    if isinstance(self._root_ref, weakref.ref):
      derefed = self._root_ref()
      assert derefed is not None
      return derefed
    else:
      return self._root_ref

  def _breadth_first_traversal(self):
    """Find shortest paths to all dependencies of self.root."""
    bfs_sorted = []
    to_visit = collections.deque([self.root])
    node_paths = object_identity.ObjectIdentityDictionary()
    node_paths[self.root] = ()
    while to_visit:
      current_trackable = to_visit.popleft()
      bfs_sorted.append(current_trackable)
      for name, dependency in self.list_children(current_trackable):
        if dependency not in node_paths:
          node_paths[dependency] = (
              node_paths[current_trackable] + (
                  base.TrackableReference(name, dependency),))
          to_visit.append(dependency)
    return bfs_sorted, node_paths

  def _add_attributes_to_object_graph(
      self, trackable_objects, object_graph_proto, node_ids, object_names,
      object_map, call_with_mapped_captures):
    """Create saveables/savers and corresponding protos in the object graph."""
    # The loop below creates TrackableObject protos in the TrackableObjectGraph,
    # which are filled in the `_add_attributes_to_object_graph_for_*` methods.
    for checkpoint_id, (trackable, unused_object_proto) in enumerate(
        zip(trackable_objects, object_graph_proto.nodes)):
      assert node_ids[trackable] == checkpoint_id

    checkpoint_factory_map, registered_savers = (
        get_checkpoint_factories_and_keys(object_names, object_map))

    # Add attributes, which describe what values are saved in checkpoint for
    # this trackable.
    _add_attributes_to_object_graph_for_registered_savers(
        registered_savers, object_graph_proto, node_ids)
    named_saveable_objects, feed_additions = (
        self._add_attributes_to_object_graph_for_saveable_objects(
            checkpoint_factory_map, object_graph_proto, node_ids, object_map,
            call_with_mapped_captures))
    return named_saveable_objects, feed_additions, registered_savers

  def _add_attributes_to_object_graph_for_saveable_objects(
      self, checkpoint_factory_map, object_graph_proto, node_ids, object_map,
      call_with_mapped_captures):
    """Create SaveableObjects and corresponding SerializedTensor protos."""
    named_saveable_objects = []
    if self._saveables_cache is None:
      # No SaveableObject caching. Either we're executing eagerly, or building a
      # static save which is specialized to the current Python state.
      feed_additions = None
    else:
      # If we are caching SaveableObjects, we need to build up a feed_dict with
      # functions computing volatile Python state to be saved with the
      # checkpoint.
      feed_additions = {}
    for trackable, factory_data_list in checkpoint_factory_map.items():
      object_proto = object_graph_proto.nodes[node_ids[trackable]]
      if self._saveables_cache is not None:
        object_to_save = _get_mapped_trackable(trackable, object_map)
        cached_attributes = self._saveables_cache.setdefault(object_to_save, {})
      else:
        cached_attributes = None

      for factory_data in factory_data_list:
        attribute = object_proto.attributes.add()
        attribute.name = name = factory_data.name
        attribute.checkpoint_key = key = factory_data.checkpoint_key
        saveable_factory = factory_data.factory

        # See if we can skip saving this checkpoint key.
        saveables = cached_attributes.get(name) if cached_attributes else None
        if saveables is not None:
          for saveable in saveables:
            if key not in saveable.name:
              # The checkpoint key for this SaveableObject is different. We
              # need to re-create it.
              saveables = None
              del cached_attributes[name]
              break

        if saveables is None:
          if callable(saveable_factory):
            maybe_saveable = saveable_object_util.create_saveable_object(
                saveable_factory, key, call_with_mapped_captures)
          else:
            maybe_saveable = saveable_factory
          if isinstance(maybe_saveable, saveable_object_lib.SaveableObject):
            saveables = (maybe_saveable,)
          else:
            # Figure out the name-based Saver's name for this variable. If it's
            # already a SaveableObject we'd just get the checkpoint key back, so
            # we leave full_name blank.
            saver_dict = saveable_object_util.op_list_to_dict(
                [maybe_saveable], convert_variable_to_tensor=False)
            full_name, = saver_dict.keys()
            saveables = tuple(saveable_object_util.saveable_objects_for_op(
                op=maybe_saveable, name=key))
            for saveable in saveables:
              saveable.full_name = full_name
          for saveable in saveables:
            if key not in saveable.name:
              raise AssertionError(
                  f"The object {trackable} produced a SaveableObject with name "
                  f"'{saveable.name}' for attribute '{name}'. Expected a name"
                  f" containing '{key}'.")
          if cached_attributes is not None:
            cached_attributes[name] = saveables

        for saveable in saveables:
          if hasattr(saveable, "full_name"):
            attribute.full_name = saveable.full_name
          if isinstance(saveable, base.PythonStateSaveable):
            if feed_additions is None:
              assert self._saveables_cache is None
              # If we're not caching saveables, then we're either executing
              # eagerly or building a static save/restore (e.g. for a
              # SavedModel). In either case, we should embed the current Python
              # state in the graph rather than relying on a feed dict.
              saveable = saveable.freeze()
            else:
              saveable_feed_dict = saveable.feed_dict_additions()
              for new_feed_key in saveable_feed_dict.keys():
                if new_feed_key in feed_additions:
                  raise AssertionError(
                      f"The object {trackable} tried to feed a value for the "
                      f"Tensor {new_feed_key} when saving, but another object "
                      "is already feeding a value.")
              feed_additions.update(saveable_feed_dict)
          named_saveable_objects.append(saveable)

    return named_saveable_objects, feed_additions

  def _add_checkpoint_values_check(self, trackable_objects, object_graph_proto):
    """Determines which objects have checkpoint values and saves to the proto.

    Args:
      trackable_objects: A list of all trackable objects.
      object_graph_proto: A `TrackableObjectGraph` proto.
    """
    # Trackable -> set of all trackables that depend on it (the "parents").
    # If a trackable has checkpoint values, then all of the parents can be
    # marked as having checkpoint values.
    parents = object_identity.ObjectIdentityDictionary()
    checkpointed_trackables = object_identity.ObjectIdentitySet()

    # First pass: build dictionary of parent objects and initial set of
    # checkpointed trackables.
    for trackable, object_proto in zip(trackable_objects,
                                       object_graph_proto.nodes):
      if (object_proto.attributes or object_proto.slot_variables or
          object_proto.HasField("registered_saver")):
        checkpointed_trackables.add(trackable)
      for child_proto in object_proto.children:
        child = trackable_objects[child_proto.node_id]
        if child not in parents:
          parents[child] = object_identity.ObjectIdentitySet()
        parents[child].add(trackable)

    # Second pass: add all connected parents to set of checkpointed trackables.
    to_visit = object_identity.ObjectIdentitySet()
    to_visit.update(checkpointed_trackables)

    while to_visit:
      trackable = to_visit.pop()
      if trackable not in parents:
        # Some trackables may not have parents (e.g. slot variables).
        continue
      current_parents = parents.pop(trackable)
      checkpointed_trackables.update(current_parents)
      for parent in current_parents:
        if parent in parents:
          to_visit.add(parent)

    for node_id, trackable in enumerate(trackable_objects):
      object_graph_proto.nodes[node_id].has_checkpoint_values.value = bool(
          trackable in checkpointed_trackables)

  def _fill_object_graph_proto(self, trackable_objects,
                               node_ids,
                               slot_variables,
                               object_graph_proto=None):
    """Name non-slot `Trackable`s and add them to `object_graph_proto`."""
    if object_graph_proto is None:
      object_graph_proto = (
          trackable_object_graph_pb2.TrackableObjectGraph())
    for checkpoint_id, trackable in enumerate(trackable_objects):
      assert node_ids[trackable] == checkpoint_id
      object_proto = object_graph_proto.nodes.add()
      object_proto.slot_variables.extend(slot_variables.get(trackable, ()))
      for child in self.list_children(trackable):
        child_proto = object_proto.children.add()
        child_proto.node_id = node_ids[child.ref]
        child_proto.local_name = child.name
    return object_graph_proto

  def _serialize_gathered_objects(self, trackable_objects, node_paths,
                                  object_map=None,
                                  call_with_mapped_captures=None):
    """Create SaveableObjects and protos for gathered objects."""
    object_names = object_identity.ObjectIdentityDictionary()
    for obj, path in node_paths.items():
      object_names[obj] = trackable_utils.object_path_to_string(path)
    node_ids = object_identity.ObjectIdentityDictionary()
    for node_id, node in enumerate(trackable_objects):
      node_ids[node] = node_id
    slot_variables = _serialize_slot_variables(
        trackable_objects=trackable_objects,
        node_ids=node_ids,
        object_names=object_names)
    object_graph_proto = self._fill_object_graph_proto(
        trackable_objects=trackable_objects,
        node_ids=node_ids,
        slot_variables=slot_variables)
    named_saveable_objects, feed_additions, registered_savers = (
        self._add_attributes_to_object_graph(
            trackable_objects=trackable_objects,
            object_graph_proto=object_graph_proto,
            node_ids=node_ids,
            object_names=object_names,
            object_map=object_map,
            call_with_mapped_captures=call_with_mapped_captures))
    # Gather all trackables that have checkpoint values or descendants with
    # checkpoint values, and add that info to the proto.
    self._add_checkpoint_values_check(trackable_objects, object_graph_proto)
    return (named_saveable_objects, object_graph_proto, feed_additions,
            registered_savers)

  def serialize_object_graph(self):
    """Determine checkpoint keys for variables and build a serialized graph.

    Non-slot variables are keyed based on a shortest path from the root saveable
    to the object which owns the variable (i.e. the one which called
    `Trackable._add_variable` to create it).

    Slot variables are keyed based on a shortest path to the variable being
    slotted for, a shortest path to their optimizer, and the slot name.

    Returns:
      A tuple of (named_variables, object_graph_proto, feed_additions):
        named_variables: A dictionary mapping names to variable objects.
        object_graph_proto: A TrackableObjectGraph protocol buffer
          containing the serialized object graph and variable references.
        feed_additions: A dictionary mapping from Tensors to values which should
          be fed when saving.

    Raises:
      ValueError: If there are invalid characters in an optimizer's slot names.
    """
    named_saveable_objects, object_graph_proto, feed_additions, _ = (
        self.serialize_object_graph_with_registered_savers())
    return named_saveable_objects, object_graph_proto, feed_additions

  def serialize_object_graph_with_registered_savers(self):
    """Determine checkpoint keys for variables and build a serialized graph."""
    trackable_objects, node_paths = self._breadth_first_traversal()
    return self._serialize_gathered_objects(
        trackable_objects, node_paths)

  def frozen_saveable_objects(self, object_map=None, to_graph=None,
                              call_with_mapped_captures=None):
    """Creates SaveableObjects with the current object graph frozen."""
    return self.frozen_saveables_and_savers(object_map, to_graph,
                                            call_with_mapped_captures)[0]

  def frozen_saveables_and_savers(self, object_map=None, to_graph=None,
                                  call_with_mapped_captures=None):
    """Generates SaveableObjects and registered savers in the frozen graph."""
    trackable_objects, node_paths = self._breadth_first_traversal()
    if to_graph:
      target_context = to_graph.as_default
    else:
      target_context = ops.NullContextmanager
    with target_context():
      named_saveable_objects, graph_proto, _, registered_savers = (
          self._serialize_gathered_objects(trackable_objects,
                                           node_paths,
                                           object_map,
                                           call_with_mapped_captures))
      with ops.device("/cpu:0"):
        object_graph_tensor = constant_op.constant(
            graph_proto.SerializeToString(), dtype=dtypes.string)
      named_saveable_objects.append(
          base.NoRestoreSaveable(
              tensor=object_graph_tensor,
              name=base.OBJECT_GRAPH_PROTO_KEY))
    return named_saveable_objects, registered_savers

  def objects_ids_and_slot_variables_and_paths(self):
    """Traverse the object graph and list all accessible objects.

    Looks for `Trackable` objects which are dependencies of
    `root_trackable`. Includes slot variables only if the variable they are
    slotting for and the optimizer are dependencies of `root_trackable`
    (i.e. if they would be saved with a checkpoint).

    Returns:
      A tuple of (trackable objects, paths from root for each object,
                  object -> node id, slot variables, object_names)
    """
    trackable_objects, node_paths = self._breadth_first_traversal()
    object_names = object_identity.ObjectIdentityDictionary()
    for obj, path in node_paths.items():
      object_names[obj] = trackable_utils.object_path_to_string(path)
    node_ids = object_identity.ObjectIdentityDictionary()
    for node_id, node in enumerate(trackable_objects):
      node_ids[node] = node_id
    slot_variables = _serialize_slot_variables(
        trackable_objects=trackable_objects,
        node_ids=node_ids,
        object_names=object_names)
    return (trackable_objects, node_paths, node_ids, slot_variables,
            object_names)

  def objects_ids_and_slot_variables(self):
    trackable_objects, _, node_ids, slot_variables, _ = (
        self.objects_ids_and_slot_variables_and_paths())
    return trackable_objects, node_ids, slot_variables

  def list_objects(self):
    """Traverse the object graph and list all accessible objects."""
    trackable_objects, _, _ = self.objects_ids_and_slot_variables()
    return trackable_objects



# Copyright 2016 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Lowers list comprehensions into for and if statements.

Example:

  result = [x * x for x in xs]

becomes

  result = []
  for x in xs:
    elt = x * x
    result.append(elt)
"""

import gast

from tensorflow.python.autograph.core import converter
from tensorflow.python.autograph.pyct import templates


# TODO(mdan): This should covert directly to operator calls.


class ListCompTransformer(converter.Base):
  """Lowers list comprehensions into standard control flow."""

  def visit_Assign(self, node):
    if not isinstance(node.value, gast.ListComp):
      return self.generic_visit(node)
    if len(node.targets) > 1:
      raise NotImplementedError('multiple assignments')

    target, = node.targets
    list_comp_node = node.value

    template = """
      target = []
    """
    initialization = templates.replace(template, target=target)

    template = """
      target.append(elt)
    """
    body = templates.replace(template, target=target, elt=list_comp_node.elt)

    for gen in reversed(list_comp_node.generators):
      for gen_if in reversed(gen.ifs):
        template = """
          if test:
            body
        """
        body = templates.replace(template, test=gen_if, body=body)
      template = """
        for target in iter_:
          body
      """
      body = templates.replace(
          template, iter_=gen.iter, target=gen.target, body=body)

    return initialization + body


def transform(node, ctx):
  return ListCompTransformer(ctx).visit(node)
